/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.action;

import java.io.InputStream;

import javax.servlet.http.HttpSession;

import org.loom.i18n.Message;
import org.loom.i18n.MessagesRepository;
import org.loom.mapping.Event;
import org.loom.persistence.file.PersistentFile;
import org.loom.resolution.ForwardResolution;
import org.loom.resolution.InputStreamResolution;
import org.loom.resolution.JacksonResolution;
import org.loom.resolution.LocaleForwardResolutionImpl;
import org.loom.resolution.PersistentFileResolution;
import org.loom.resolution.RedirectResolution;
import org.loom.resolution.StringResolution;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.names.ContentTypeNames;
import org.loom.util.StringUtils;

/**
 * Abstract superclass that provides some methods for the most common use.
 * @author icoloma
 */
public abstract class AbstractAction implements Action { 

	private ActionContext context;
	
	public void setContext(ActionContext context) {
		this.context = context;
	}
	
	public MessagesRepository getMessagesRepository() {
		return context.getRequest().getMessagesRepository();
	}
	
	/**
	 * @return true if there is any validation/conversion error
	 */
	public boolean hasAnyError() {
		return getRequest().getParsedAction().hasAnyError();
	}
	
	/**
	 * @param propertyPath the property path to search error for
	 * @return true if there are validation/conversion errors associated to the provided property path
	 */
	public boolean hasAnyError(String propertyPath) {
		return getRequest().getParsedAction().hasAnyError(propertyPath);
	}
	
	/**
	 * Forwards to the provided JSP page name.  
	 */
	protected ForwardResolution forward(String pageName) {
		return context.getResolutionFactory().forward(pageName);
	}
	
	/**
	 * Forward to a page, with preference for a localized page if possible.
	 * A forward to "myfile.jsp" with spanish request locale and english default locale 
	 * will first try "myfile_es.jsp", then "myfile_en.jsp" and, if none is found, "myfile.jsp"
	 * @see LocaleForwardResolutionImpl
	 * @param pageName the name of the resource to forward to
	 * @return a ForwardResolution to the provided page
	 */
	protected ForwardResolution forwardWithLocale(String pageName) {
		return context.getResolutionFactory().forwardWithLocale(pageName);
	}
	
	/**
	 * Forwards to a page name with the same name of the current event  
	 */
	protected ForwardResolution forward() {
		Event event = getContext().getEvent();
		String eventName = event.getName();
		return forward(event.getActionMapping().getHandle() + "/" + StringUtils.dasherize(eventName) + ".jsp");
	}
	
	/**
	 * Redirects to the provided url 
	 */
	protected RedirectResolution redirect(String uri) {
		return context.getResolutionFactory().redirect(uri);
	}
	
	/**
	 * Redirects to the provided action and event. 
	 */
	protected RedirectResolution redirect(Class<? extends Action> actionClass, String event) {
		return context.getResolutionFactory().redirect(actionClass, event);
	}
	
	/**
	 * Redirects to the provided action, default event
	 */
	protected RedirectResolution redirect(Class<? extends Action> actionClass) {
		return context.getResolutionFactory().redirect(actionClass, null);
	}
	
	/**
	 * Redirects to the provided action and event. The action parameter can be the short or full action name
	 */
	protected RedirectResolution redirect(String action, String event) {
		return context.getResolutionFactory().redirect(action, event);
	}

	/**
	 * Send a file to the browser. 
	 */
	protected PersistentFileResolution send(PersistentFile file) {
		return context.getResolutionFactory().send(file);
	}
	
	/**
	 * Send a file to the browser. 
	 */
	protected InputStreamResolution send(String filename, InputStream input) {
		return new InputStreamResolution(filename, input);
	}
	
	/**
	 * Return a simple text as the response. The mime type will be set as text/plain by default.
	 * @param results the result to be handled to the browser as is.
	 */
	protected StringResolution send(String results) {
		StringResolution resolution = new StringResolution(results);
		resolution.setContentType(ContentTypeNames.TEXT);
		return resolution;
	}
	
	/**
	 * Returns the value of the given property as a UrlParameter to
	 * be added to a Resolution
	 */
	protected String getPropertyAsString(String propertyPath) {
		return getRequest().getParsedAction().getPropertyAsString(propertyPath);
	}
	
	/**
	 * @deprecated use getPropertyAsString instead
	 */
	protected String getParam(String propertyPath) {
		return getPropertyAsString(propertyPath);
	}

	public ActionContext getContext() {
		return context;
	}
	
	/**
	 * @return the request object that is being processed
	 */
	public LoomServletRequest getRequest() {
		return context.getRequest();
	}
	
	/**
	 * @return the response object that will be returned
	 */
	public LoomServletResponse getResponse() {
		return context.getResponse();
	}
	
	public HttpSession getSession() {
		return getRequest().getSession();
	}
	
	public HttpSession getSession(boolean create) {
		return getRequest().getSession(create);
	}
	
	/**
	 * @deprecated use error() instead
	 */
	public Message addError(String normalizedPropertyPath, String messageKey){
		return error(normalizedPropertyPath, messageKey);
	}
	

	/**
	 * @deprecated use error() instead
	 */
	public Message addError( String messageKey){
		return error(messageKey);
	}
	
	/**
	 * @deprecated use addMessage() instead
	 */
	public Message addError( Message message){
		return context.getMessages().addMessage(message);
	}
	
	public Message addMessage( Message message){
		return context.getMessages().addMessage(message);
	}
	
	protected Message error(String propertyPath, String messageKey) {
		return context.getMessages().error(propertyPath, messageKey);
	}
	
	protected Message error(String messageKey) {
		return context.getMessages().error(messageKey);
	}
	
	protected Message info(String messageKey) {
		return context.getMessages().info(messageKey);
	}
	
	protected Message warn(String messageKey) {
		return context.getMessages().warn(messageKey);
	}
	
	protected JacksonResolution json(Object contents) {
		return context.getResolutionFactory().json(contents);
	}
	
}
