package org.loom.annotation.processor;

import org.loom.exception.ConfigException;
import org.loom.interceptor.Interceptor;
import org.loom.validator.PropertyValidator;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;

/**
 * {@link AnnotationProcessor} that uses {@link PropertyValidator} or {@link Interceptor} instances injected using spring.
 * This processor will ask spring for a new instance each time getInstance() gets called, 
 * which is once for each Loom annotation found.  
 * The lookup will be delayed until the annotation gets processed, so any processor subclass can be registered with 
 * loom, but the bean instance will not be created until the annotation is effectively used. 
 * 
 * @author icoloma
 */
public abstract class AbstractInjectedAnnotationProcessor<T> extends AbstractAnnotationProcessor implements ApplicationContextAware {
	
	/** the interceptor bean name */
	private String beanName;
	
	/** the spring {@link ApplicationContext} instance */ 
	private ApplicationContext applicationContext;

	/** the class of the injected interceptor */
	private Class<T> beanClass;
	
	public AbstractInjectedAnnotationProcessor( Class<T> beanClass) {
		this.beanClass = beanClass;
	}
	
	@SuppressWarnings("unchecked")
	protected T getInstance() {
		if (beanName == null) {
			throw new ConfigException("There is no configured bean of type " + beanClass.getName());
		}
		return (T) applicationContext.getBean(beanName);
	}

	public void setApplicationContext(ApplicationContext applicationContext) {
		this.applicationContext = applicationContext;
		String[] names = applicationContext.getBeanNamesForType(beanClass);
		if (names.length > 0) {
			this.beanName = names[0];
		} 
	}
	
	public String getBeanName() {
		return beanName;
	}
	
	public void setBeanName(String interceptorBeanName) {
		this.beanName = interceptorBeanName;
	}

	public ApplicationContext getApplicationContext() {
		return applicationContext;
	}

	public Class<T> getBeanClass() {
		return beanClass;
	}
	
}
