/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArraySet;

import org.loom.action.Action;
import org.loom.exception.PropertyNotFoundException;
import org.loom.i18n.Messages;
import org.loom.i18n.MessagesRepository;
import org.loom.log.Log;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.util.PropertyUtils;


/**
 * A map that handles a group of PropertyBinder together
 * This class is thread safe
 * @author icoloma
 *
 */
public class PropertyBinderMap {
	
	/** class that is the root of any property path */
	private Class<?> containerClass;
	
	/** properties can be added by any thread, but are mostly read-only */
	private Map<String, PropertyBinder> binders = new ConcurrentHashMap<String, PropertyBinder>();
	
	/** 
	 * the list of parameters that are not properties and should not be 
	 * bound if found.
	 * Once a property has been asked once and not found, it gets added to this set */
	protected Set<String> unboundParameters = new CopyOnWriteArraySet<String>();
	
	private PropertyBinderFactory propertyBinderFactory;
	
	private static Log log = Log.getLog();
	
	/**
	 * Same as getPropertyBinder, but the property is required to exist and will throw an exception
	 * if it does not.
	 */
	public PropertyBinder getRequiredPropertyBinder(String normalizedPropertyPath) {
		PropertyBinder binder = getPropertyBinder(normalizedPropertyPath);
		if (binder == null) {
			throw new IllegalArgumentException("Property " + containerClass.getSimpleName() + "." + normalizedPropertyPath + " not found");
		}
		return binder;

	}
	/**
	 * Asks for the PropertyBinding instance corresponding to a given propertyPath.
	 * If it exists, return it. If it does not yet exist, create and return it.
	 * If it cannot be created, register the propertyPath as unbound so it will not
	 * try to create it again in the future
	 * @return the corresponding PropertyBinding, null if none
	 */
	public PropertyBinder getPropertyBinder(String normalizedPropertyPath) {
		if (unboundParameters.contains(normalizedPropertyPath)) {
			return null;
		}
		PropertyBinder result = binders.get(normalizedPropertyPath);
		if (result != null) {
			return result;
		}
		try {
			result = propertyBinderFactory.create(containerClass, normalizedPropertyPath);
			binders.put(normalizedPropertyPath, result);
			return result;
		} catch (PropertyNotFoundException e) {
			log.debug("Could not resolve property path ", normalizedPropertyPath, " for action ", containerClass.getSimpleName(), 
					". This property will be ignored in the future.");
			unboundParameters.add(normalizedPropertyPath);
			return null;
		}
	}
	
	/**
	 * @return the class of the specified property
	 * @throws PropertyNotFoundException if the property could not be found
	 */
	public Class<?> getPropertyClass(String normalizedPropertyPath) {
		PropertyBinder binder = getRequiredPropertyBinder(normalizedPropertyPath);
		return binder.getPropertyClass();
	}
	
	/**
	 * Applies the given values to the object passed as argument.
	 * This method accepts property values as Strings or FileItem (uploaded files)
	 */
	public void populate(Action action, ServletRequestParameters parameters, MessagesRepository messagesRepository, Messages messages) {
		for (Map.Entry<String, String> entry : parameters.getStringParametersEntrySet()) {
			String paramName = entry.getKey();
			if (!RequestParameterNames.isInternal(paramName) && !parameters.isAssigned(paramName)) {
				PropertyBinder binder = getPropertyBinder(PropertyUtils.normalize(paramName));
				if (binder != null) {
					binder.setValue(action, paramName, entry.getValue(), messages, messagesRepository);
				}
				parameters.setAssigned(paramName);
			}
		}
	}

	public int size() {
		return binders.size();
	}

	/**
	 * @return the values for the specified path inside the container object
	 */
	public Object getValue(Object container, String propertyPath) {
		PropertyBinder binder = getRequiredPropertyBinder(PropertyUtils.normalize(propertyPath));
		return binder.getValue(container, propertyPath);
	}
	
	/**
	 * @return the values for the specified path inside the container object
	 */
	public String getValueAsText(Object container, String propertyPath, MessagesRepository repository) {
		PropertyBinder binder = getRequiredPropertyBinder(PropertyUtils.normalize(propertyPath));
		return binder.getValueAsText(container, propertyPath, repository);
	}

	public void setPropertyBinderFactory(PropertyBinderFactory propertyBinderFactory) {
		this.propertyBinderFactory = propertyBinderFactory;
	}

	public void setContainerClass(Class<?> containerClass) {
		this.containerClass = containerClass;
	}

	public Map<String, PropertyBinder> getBinders() {
		return binders;
	}

	public Set<String> getUnboundParameters() {
		return unboundParameters;
	}
	public Class<?> getContainerClass() {
		return containerClass;
	}

}
