/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.binding;

import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.WeakHashMap;

import org.apache.commons.lang.ArrayUtils;
import org.loom.binding.factories.ImplementationFactoriesRepository;
import org.loom.binding.factories.ImplementationFactory;
import org.loom.exception.ConfigException;
import org.loom.exception.PropertyNotFoundException;
import org.loom.util.ClassUtils;

import com.google.common.collect.Lists;
import com.google.common.collect.Sets;


public class PropertyWrapperFactory {

	/** the list of configured Implementation Factories */
	private ImplementationFactoriesRepository implementationFactoriesRepository;
	
	private ItemMetadataFactory itemMetadataFactory;
	
	private WeakHashMap<Class<?>, List<PropertyWrapper>> classMetadataCache = new WeakHashMap<Class<?>, List<PropertyWrapper>>();
	
	public PropertyWrapperFactory() {
		implementationFactoriesRepository = new ImplementationFactoriesRepository();
		itemMetadataFactory = new ItemMetadataFactory();
		itemMetadataFactory.setImplementationFactoriesRepository(implementationFactoriesRepository);
	}
	
	public void cleanCache(Class<?> clazz) {
		classMetadataCache.remove(clazz);
	}
	
	/**
	 * @param containerClass the container class
	 * @param propertyName the name of the property
	 */
	public PropertyWrapper createInstance(Class<?> containerClass, String propertyName) {
		
		try {
			Field field = ClassUtils.getField(containerClass, propertyName);
			Method getter = ClassUtils.getGetter(containerClass, propertyName);
			Method setter = ClassUtils.getSetter(containerClass, propertyName);
			
			if (field == null && getter == null && setter == null) {
				throw new PropertyNotFoundException(containerClass, propertyName);
			}
			
			// instantiate the wrapper
			PropertyWrapper property = new PropertyWrapper(containerClass, propertyName);
			property.setField(field);
			property.setGetter(getter);
			property.setSetter(setter);
			
			// assign the nullValue, if needed
			if (Integer.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Integer.valueOf(0));
			} else if (Double.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Double.valueOf(0));
			} else if (Long.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Long.valueOf(0));
			} else if (Short.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Short.valueOf((short)0));
			} else if (Character.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Character.valueOf('\0'));
			} else if (Byte.TYPE.equals(property.getPropertyClass())) {
				property.setNullValue(Byte.valueOf((byte)0));
			}
			
			// assign the implementationFactory for the property
			if (!property.isReadOnly()) {
				ImplementationFactory<?> implementationFactory = implementationFactoriesRepository.getImplementationFactory(property);
				if (implementationFactory != null) {
					property.setImplementationFactory(implementationFactory);
				}
			}
			
			// if the property is Map, Collection or Array, calculate the itemMetadata to apply
			property.setItemMetadata(createItemMetadata(property));
			
			return property;
		} catch (ConfigException e) {
			throw new ConfigException("Could not create propertyWrapper for " + containerClass.getSimpleName() + "." + propertyName, e);
		}

	}

	public ItemMetadata[] createItemMetadata(AnnotationContainer container) {
		return itemMetadataFactory.createItemMetadata(container);
	}

	/**
	 * Return the list of properties with the provided annotation
	 */
	public List<PropertyWrapper> getAnnotatedProperties(Class<?> clazz, Class<? extends Annotation> annotationClass, String[] inclProp, String[] exclProp) {
		List<PropertyWrapper> properties = classMetadataCache.get(clazz);
		if (properties == null) {
			synchronized (this) {
				properties = Lists.newArrayList();
				properties = getProperties(clazz, properties, new HashSet<String>());
				classMetadataCache.put(clazz, properties);
			}
		}
		
		Set<String> excludedProperties = exclProp == null? new HashSet<String>() : Sets.newHashSet(exclProp);
		Set<String> includedProperties = ArrayUtils.isEmpty(inclProp)? null : Sets.newHashSet(inclProp);
		// filter only properties with the specified annotation
		List<PropertyWrapper> annotatedProperties = Lists.newArrayListWithCapacity(properties.size());
		for (PropertyWrapper propertyWrapper : properties) {
			if (propertyWrapper.getAnnotation(annotationClass) != null && 
					!excludedProperties.contains(propertyWrapper.getPropertyName()) &&
					(includedProperties == null || includedProperties.contains(propertyWrapper.getPropertyName()))
					) {
				annotatedProperties.add(propertyWrapper);
			}
		}
		return annotatedProperties;
	}
	
	private List<PropertyWrapper> getProperties(Class<?> clazz, List<PropertyWrapper> properties, Set<String> visitedProperties) {
		try {
			// add standard javabean properties
			PropertyDescriptor[] descriptors = Introspector.getBeanInfo(clazz).getPropertyDescriptors();
			for (PropertyDescriptor property : descriptors) {
				String name = property.getName();
				if (!visitedProperties.contains(name) && shouldVisit(property)) {
					visitedProperties.add(name);
					PropertyWrapper wrapper = createInstance(clazz, name);
					properties.add(wrapper);
				}
			}
			
			// add private properties that could be forced public
			for (Field field : clazz.getDeclaredFields()) {
				String fieldName = field.getName();
				if (!visitedProperties.contains(fieldName) && !Modifier.isStatic(field.getModifiers())) {
					visitedProperties.add(fieldName);
					PropertyWrapper wrapper = createInstance(clazz, fieldName);
					properties.add(wrapper);
				}
			}
			
			// add superclass declarations
			Class<?> superclass = clazz.getSuperclass();
			if (superclass != null && !Object.class.equals(superclass)) {
				return getProperties(superclass, properties, visitedProperties);
			}			
			return properties;
			
		} catch (IntrospectionException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * @return true if there is any annotation on this property accesor methods
	 */
	private boolean shouldVisit(PropertyDescriptor property) {
		
		boolean result = false;
		Method readMethod = property.getReadMethod();
		if (readMethod != null) {
			result = !Modifier.isStatic(readMethod.getModifiers()); 
		}
		Method writeMethod = property.getWriteMethod();
		if (writeMethod != null) {
			result = result || !Modifier.isStatic(writeMethod.getModifiers()); 
		}
		return result;
	}
	
	public void setImplementationFactoriesRepository(ImplementationFactoriesRepository implementationFactories) {
		this.implementationFactoriesRepository = implementationFactories;
	}

}
