package org.loom.config;

import java.util.Collection;
import java.util.Iterator;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Delegate into a sequential series of LocaleResolver.
 * This is useful for combining a LocaleResolver to use in production stage (e.g. 
 * {@link PrefixLocaleResolver}) with another to use in development.
 * @author icoloma
 *
 */
public class CompositeLocaleResolver implements LocaleResolver {

	/** the list of configured localeResolver */
	private Collection<LocaleResolver> resolvers;
	
	/**
	 * Invoke each configured resolver sequentially until one
	 * returns a not-null result. 
	 * @return the first non-null result obtained from a configured resolver
	 */
	public Locale getLocale(HttpServletRequest request) {
		Locale result = null;
		for (Iterator<LocaleResolver> i = resolvers.iterator(); i.hasNext() && result == null; ) {
			result = i.next().getLocale(request);
		}
		return result;
	}

	/**
	 * Invoke each configured resolver sequentially until one
	 * does not throw a {@link UnsupportedOperationException}
	 * @throws UnsupportedOperationException if none of the configured resolvers is able to set the locale
	 */
	public void setLocale(HttpServletResponse response, Locale locale) {
		for (LocaleResolver resolver : resolvers) {
			try {
				resolver.setLocale(response, locale);
				return;
			} catch (UnsupportedOperationException e) {
				// ignore
			}
		}
		throw new UnsupportedOperationException();
	}

	public void setResolvers(Collection<LocaleResolver> resolvers) {
		this.resolvers = resolvers;
	}

}
