package org.loom.config;

import java.util.Iterator;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.loom.i18n.MessagesRepository;
import org.loom.i18n.MessagesRepositoryEventListener;
import org.loom.log.Log;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.CustomizableThreadFactory;

/**
 * Keeps periodic monitorization of configured Resource files. 
 * To start, just add it to your spring config file:
 * <pre>
      &lt;bean class="org.loom.config.ResourcesWatchdog" />
 * </pre>
 * 
 * @author icoloma
 *
 */
public class ResourcesWatchdog implements Runnable, InitializingBean {

	@Autowired
	private Config config;
	
	/** the number of seconds to sleep between monitorization intervals, default to 3 */
	private int sleepIntervalInSeconds = 3;
	
	/** true to be enabled, false otherwise. Defaults to the same value of Config.development */
	private Boolean enabled;
	
	/** the list of observers. Note that new observers may be added by multiple threads */
	private List<ResourcesObserver> observers = new CopyOnWriteArrayList<ResourcesObserver>();
	
	/** the Executor instance that handles periodic execution */
	private ScheduledExecutorService executor;
	
	private static Log log = Log.getLog();
	
	public void afterPropertiesSet() throws Exception {
		if (enabled == null) {
			enabled = config.isDevelopment();
		}
		addListener(config.getWebResourcesRepository());
		config.getMessagesRepositoryFactory().addMessagesRepositoryEventListener(new MessagesRepositoryEventListener() {

			public void onCreate(MessagesRepository repository) {
				ResourcesWatchdog.this.addListener(repository);
			}
			
		});
		Config.addUndeployTask(new Runnable() {
			public void run() {
				// stop the resourceModificationWatchdog 
				try {
					ResourcesWatchdog.this.stopMonitorization();
				} catch (Exception e) {
					System.err.println(e.getMessage());
				}
			}
		});
		if (enabled) {
			this.startMonitorization();
		}
	}
	
	/**
	 * Start a periodic thread to monitor resources for modification
	 */
	public void startMonitorization() {
		if (executor == null) {
			log.info("Starting ResourceWatchdog");
			executor = Executors.newScheduledThreadPool(1, new CustomizableThreadFactory("Loom Resources Watchdog "));
			executor.scheduleAtFixedRate(this, sleepIntervalInSeconds, sleepIntervalInSeconds, TimeUnit.SECONDS);
		}
	}
	
	/**
	 * Stop a monitoring resources 
	 */
	public void stopMonitorization() {
		if (executor != null && !executor.isShutdown()) {
			log.info("Stopping ResourceWatchdog");
			executor.shutdownNow();
			executor = null;
		}
	}
	
	public void run() {
		try {
			for (ResourcesObserver observer : observers) {
				observer.checkResources();
			}
		} catch (Exception e) {
			// do not forward it, as it would prevent further watchdog execution
			log.error(e, e);
		}
	}
	
	public void addListener(ResourcesModificationListener listener) {
		observers.add(new ResourcesObserver(listener));
	}
	
	public void removeListener(ResourcesModificationListener listener) {
		for (Iterator<ResourcesObserver> i = observers.iterator(); i.hasNext();) {
			ResourcesObserver observer = i.next();
			if (observer.getListener() == listener) {
				i.remove();
			}
		}
	}
	
	/** 
	 * Remove all observers of the specified class
	 */
	public void removeAll(Class<? extends ResourcesObserver> clazz) {
		for (Iterator<ResourcesObserver> i = observers.iterator(); i.hasNext(); ) {
			if (clazz.isAssignableFrom(i.next().getClass())) {
				i.remove();
			}
		}
	}

	public void setEnabled(Boolean enabled) {
		this.enabled = enabled;
	}

	public void setSleepIntervalInSeconds(int sleepIntervalInSeconds) {
		this.sleepIntervalInSeconds = sleepIntervalInSeconds;
	}
	
}
