/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.filter;

import static org.loom.servlet.names.WebContextParameterNames.CONFIG_FACTORY_CLASS;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.loom.config.Config;
import org.loom.config.ConfigFactory;
import org.loom.config.SpringConfigFactory;
import org.loom.log.Log;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletResponse;
import org.loom.servlet.Workflow;
import org.loom.util.ClassUtils;


/**
 * The registered loom filter.
 * @author Ignacio Coloma
 */
public class LoomFilter extends AbstractLoomFilter {
	
	/** servlet init parameter. Set to true to initialize somewhere else */
	private static final String INITIALIZE_PARAM = "initialize";

	/** the workflow instance. This class does all the work. */
	private static Workflow workflow;
	
	private static Log log = Log.getLog();
	
	@Override
	protected void doFilter(LoomServletRequest request, LoomServletResponse response, FilterChain chain) throws IOException, ServletException {
		workflow.doFilter(request, response,  chain);
	}
	
	/**
	 * Create and configure the config instance
	 */
	@Override
	public void init(FilterConfig fc) throws ServletException {
		ServletContext context = fc.getServletContext();
		String init = fc.getInitParameter(INITIALIZE_PARAM);
		if (init == null || Boolean.parseBoolean(init)) {
			initialize(context);
		}
	}
	
	public static void initialize(ServletContext context) throws ServletException {
		
		try {
			String configFactoryClassName = context.getInitParameter(CONFIG_FACTORY_CLASS);
			ConfigFactory factory = configFactoryClassName == null? new SpringConfigFactory() : (ConfigFactory) ClassUtils.newInstance(Class.forName(configFactoryClassName));
			log.debug("Loading config from class " + factory.getClass().getName());
			Config config = factory.createConfig(context);
			config.setServletContext(context);
			Config.setInstance(config);
			config.validate();
			config.scanAndConfigureActions();
			workflow = new Workflow(context);
			workflow.setResolutionFactory(config.getResolutionFactory());
			workflow.setActionMappingRepository(config.getActionMappingRepository());
			workflow.setExceptionHandler(config.getExceptionHandler());
			
			// add the config as an application-scoped variable
			context.setAttribute("config", config);
			
		} catch (ClassNotFoundException e) {
			throw new ServletException(e);
		}
	}
	
	@Override
	public void destroy() {
		Config.cleanup();
	}
	
}

