/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.i18n;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.concurrent.ConcurrentHashMap;

import org.loom.util.LocaleUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.support.ResourcePatternResolver;

import com.google.common.collect.Lists;

/**
 * Default MessagesRepositoryFactory implementation.
 * Searches for the specified properties files in the classpath.
 * @author icoloma
 */
public class MessagesRepositoryFactoryImpl implements MessagesRepositoryFactory, ResourceLoaderAware, InitializingBean {

	/** the map of already instantiated repositories */
	private Map<Locale, MessagesRepository> repositories = new ConcurrentHashMap<Locale, MessagesRepository>();
	
	/** the file locations WITHOUT file extension, eg: "resources/messages" */
	private List<String> locations;
	
	/** 
	 * the message keys that should be included in the JSON representation of 
	 * any {@link MessagesRepository} 
	 */
	private Set<String> browserMessages = new TreeSet<String>();

	/** the default locale to be used, defaults to Locale.getDefault() */
	private Locale defaultLocale;
	
	/** listeners for repository creation */ 
	private List<MessagesRepositoryEventListener> listeners = new ArrayList<MessagesRepositoryEventListener>();
	
	private ResourcePatternResolver resourceResolver;

	/** the strategy to follow when looking for a message that is missing fom the ResouceBundle */
	@Autowired(required=false)
	private MissingMessageHandler missingMessagesHandler;

	public void afterPropertiesSet() {
		if (locations ==  null) {
			locations = new ArrayList<String>();
			locations.add("classpath:resources/loom-messages");
			locations.add("WEB-INF/classes/resources/messages");
		}
		if (defaultLocale == null) {
			defaultLocale = Locale.getDefault();
		}
		if (missingMessagesHandler == null) {
			missingMessagesHandler = new DefaultMissingMessageHandler();
		}
	}
	
	/**
	 * Get a repository (if already created) or creates a new one if not.
	 * The provided Locale will be compared to the properties files in the classpath, and a 
	 * {@link MessagesRepository} instance will be created for the first Locale fragment 
	 * that is a match for any properties file. 
	 * For example, if messages_es and messages_en are found in the classpath and a
	 * request for es_ES_CA is received, a MessagesRepository for "es" (the first supported Locale) 
	 * will be returned. 
	 * If not found, a Messagesrepository for defaultLocale will be returned.
	 */
	public MessagesRepository getRepository(Locale locale) {
		locale = getNearestSupportedLocale(locale);
		MessagesRepository repository = repositories.get(locale);
		if (repository == null) {
			repository = initRepository(locale);
		}
		return repository;
	}

	/**
	 * @param locale
	 * @return the list of resources (properties files) used to produce a MessagesRepository for the specified Locale.
	 * All the returned Resources are confirmed to exist.
	 */
	public List<Resource> getResources(Locale locale) {
		ArrayList<Resource> resources = new ArrayList<Resource>();
		for (String location : locations) {
			List<String> resourceNames = LocaleUtils.getLocalizedFilenames(location, ".properties", locale, defaultLocale);
			int length = resourceNames.size();
			for (int i = length - 1; i >= 0; i--) {
				String resourceName = resourceNames.get(i);
				Resource resource = resourceResolver.getResource(resourceName);
				if (resource.exists()) {
					resources.add(resource);
				}
			}
		}
		return resources;
	}
	
	/**
	 * Initialize a new repository instance.
	 * This method makes a second check: if the properties files that will be used for this locale 
	 * are the same on an already existing repository, return the existing repository instead. 
	 */
	protected synchronized MessagesRepository initRepository(Locale locale) {
		
		// typical double-if for synchronized blocks
		MessagesRepository repository = repositories.get(locale);
		if (repository != null) {
			return repository;
		}
		
		List<Resource> resources = getResources(locale);
		repository = new MessagesRepository(locale);
		repository.setResources(resources);
		repository.setMissingMessagesHandler(missingMessagesHandler);
		repository.addBrowserMessages(browserMessages);
		repository.initialize();
		repositories.put(locale, repository);
		for (MessagesRepositoryEventListener listener : listeners) {
			listener.onCreate(repository);
		}
		return repository;
			
	}

	/**
	 * Will look in the list of configured locations and return the first partial locale match
	 * @return the nearest Locale that is supported. 
	 */
	protected Locale getNearestSupportedLocale(Locale locale) {
		String language = locale.getLanguage();
		String country = locale.getCountry();
		String variant = locale.getVariant();
		for (String location : locations) {
			if (!location.endsWith("loom-messages")) { // do not consider loom-mesages.properties 
				if (variant.length() > 0 && checkResourceExists(location + "_" + locale.toString() + ".properties")) {
					return locale;
				} else if (country.length() > 0 && checkResourceExists(location + "_" + language + "_" + country + ".properties")) {
					return new Locale(language, country);
				} else if (language.length() > 0 && checkResourceExists(location + "_" + language + ".properties")) {
					return new Locale(language);
				}
			}
		}
		return defaultLocale;
	}

	/**
	 * @return true if the resource exists
	 */
	protected boolean checkResourceExists(String resourceName) {
		Resource resource = resourceResolver.getResource(resourceName);
		return resource.exists();
	}

	/**
	 * Add a message key that should be forwarded to the browser.
	 */
	public void addBrowserMessage(String messageKey) {
		if (!browserMessages.contains(messageKey)) {
			browserMessages.add(messageKey);
			
			// update existing repositories, if any
			for (MessagesRepository repo : repositories.values()) {
				repo.addBrowserMessage(messageKey);
			}
		}
	}
	
	
	public void addExtensionLocation(String location) {
		List<String> l = Lists.newArrayList();
		l.add(location);
		l.addAll(locations);
		locations = l;
	}
	
	/**
	 * Add a set of Json keys
	 */
	public void setBrowserMessages(Set<String> browserKeys) {
		for (String key : browserKeys) {
			addBrowserMessage(key);
		}
	}
	
	public Locale getDefaultLocale() {
		return defaultLocale;
	}

	public void setDefaultLocale(Locale defaultLocale) {
		this.defaultLocale = defaultLocale;
	}
	
	/**
	 * Replace this factory locations with the provided list
	 * @param locations new list of properties file locations
	 */
	public void setLocations(List<String> locations) {
		this.locations = locations;
	}

	public void setResourceLoader(ResourceLoader resourceLoader) {
		this.resourceResolver = (ResourcePatternResolver) resourceLoader;
	}

	public void addMessagesRepositoryEventListener( MessagesRepositoryEventListener listener) {
		listeners.add(listener);
	}

	public List<String> getLocations() {
		return locations;
	}

	public Set<String> getBrowserMessages() {
		return browserMessages;
	}

	public MissingMessageHandler getMissingMessagesHandler() {
		return missingMessagesHandler;
	}

	public void setMissingMessagesHandler(
			MissingMessageHandler missingMessagesHandler) {
		this.missingMessagesHandler = missingMessagesHandler;
	}


}
