/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.interceptor;

import static org.loom.config.ClassNames.SPRING_SECURITY;
import static org.loom.servlet.names.CssClassNames.UNAUTHORIZED;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.log.Log;
import org.loom.mapping.ParsedAction;
import org.loom.resolution.Resolution;
import org.loom.security.JeeSecurityAdapter;
import org.loom.security.SecurityAdapter;
import org.loom.security.SpringSecurityAdapter;
import org.loom.tags.Button;
import org.loom.tags.HtmlTag;
import org.loom.tags.Link;
import org.loom.util.ClassUtils;
import org.loom.validator.ButtonDecorator;
import org.loom.validator.LinkDecorator;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

/**
 * Checks that the user has the required roles to call the event
 * @author icoloma
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class SecurityInterceptorImpl implements SecurityInterceptor, LinkDecorator, ButtonDecorator, CreateInterceptor, InitializingBean {

	/** the class that will do the security validation */
	@Autowired(required=false)
	private SecurityAdapter securityAdapter;
	
	/** the list of roles allowed to invoke this event */
	private String[] allowedRoles;
	
	private static Log log = Log.getLog();
	
	public void afterPropertiesSet() {
		if (securityAdapter == null) {
			securityAdapter = ClassUtils.isPresent(SPRING_SECURITY) ? new SpringSecurityAdapter() : new JeeSecurityAdapter();
		}
		
	}
	
	public void decorate(Link link) {
		decorateImpl(link);
	}
	
	public void decorate(Button button) {
		decorateImpl(button);
	}
	
	/**
	 * Add the "unauthorized" css class to any link/submit button/menu item that points to this event
	 */
	private void decorateImpl(HtmlTag tag) {
		if (!securityAdapter.hasRole(allowedRoles)) {
			tag.addCssClass(UNAUTHORIZED);
		}
	}
	
	public Resolution afterCreate(ParsedAction action) {
		if (!securityAdapter.hasRole(allowedRoles)) {
			log.debug("Rejecting request because the user does not have any of the roles ", allowedRoles);
			securityAdapter.throwInsufficientPermissionsException("The user does not have enough permissions");
		}
		
		return null;
	}

	public String[] getAllowedRoles() {
		return allowedRoles;
	}

	public void setAllowedRoles(String[] allowedRoles) {
		this.allowedRoles = allowedRoles;
	}

	public void setSecurityAdapter(SecurityAdapter securityAdapter) {
		this.securityAdapter = securityAdapter;
	}

	@Override
	public String toString() {
		return new ToStringBuilder(this).append("allowedRoles", allowedRoles).toString(); 
	}

}
