/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.mapping;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.loom.action.Action;
import org.loom.exception.ActionNotFoundException;
import org.loom.log.Log;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.util.ClassUtils;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.ConfigurableListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.ConfigurableApplicationContext;

import com.google.common.collect.SetMultimap;
import com.google.common.collect.TreeMultimap;


/**
 * Central repository of all configured {@link ActionMapping} instances.
 *  
 * @author Ignacio Coloma
 */
public class ActionMappingRepository implements ApplicationContextAware, InitializingBean {

	/** all mappings by qualified className, for faster search. The action class name is the key */
	private Map<String, ActionMapping> mappingsByClassname = new HashMap<String, ActionMapping>();
	
	/** all mappings by unqualified className, for faster search. The unqualified class name is the key */
	private Map<String, ActionMapping> mappingsByUnqualifiedClassname = new HashMap<String, ActionMapping>();
	
	/** all mappings are stored here for fast lookup. The key is the handle of each action. Two actions may share the same handle */
	private SetMultimap<String, Event> eventsByHandle = TreeMultimap.create(new StringComparator(), new EventComparator());

	/** the String used to extract the request handle. Usually it is the first forward slash '/' */
	private String handleSeparator = "/"; 
	
	private static Log log = Log.getLog(); 
	
	private ActionMappingFactory actionMappingFactory;
	
	private ApplicationContext applicationContext;
	
	public void afterPropertiesSet() {
		if (actionMappingFactory == null) {
			throw new IllegalArgumentException("actionMappingFactory must be set");
		}
	}

	/**
	 * Scan and register all {@link Action} instances detected by Spring
	 */
	@SuppressWarnings("unchecked")
	public void scanAndRegisterActions() {
		ConfigurableListableBeanFactory beanFactory = ((ConfigurableApplicationContext)applicationContext).getBeanFactory();
		for (String name: beanFactory.getBeanNamesForType(Action.class)) {
			BeanDefinition bd = beanFactory.getBeanDefinition(name);
			Class<? extends Action> actionClass = (Class<? extends Action>) ClassUtils.forName(bd.getBeanClassName());
			ActionMapping mapping = actionMappingFactory.create(name, actionClass);
			addActionMapping(mapping);
		}
		validateActionMappings();
	}

	/**
	 * @param uri the uri as returned by request.getServletPath()
	 * @param request the current request object
	 * @return the ActionMapping corresponding to this request, null if no Action was a match
	 */
	public Event getEvent(String uri, LoomServletRequest request) {
		String handle = getHandle(uri);
		uri = uri.substring(handle.length());
		if (uri.length() == 0) { // is missing the end slash, will be redirected
			return null;
		}
		for (Event event : eventsByHandle.get(handle)) {
			
			Map<String, String> extraParams = event.parse(uri, request);
			if (extraParams != null) { // match
				request.getRequestParameters().addStringParameters(extraParams);
				
				// check if a submit button has overriden this event by specifying an overriding value
				for (Map.Entry<String, String> entry : request.getRequestParameters().getStringParametersEntrySet()) {
					String name = entry.getKey();
					if (RequestParameterNames.isInternal(name) && name.startsWith(RequestParameterNames.OVERRIDE_EVENT_PREFIX)) {
						String overridenEvent = name.substring(RequestParameterNames.OVERRIDE_EVENT_PREFIX.length());
						event = event.getActionMapping().getEvent(overridenEvent);
						break;
					}
				}
				
				return event;
			}
		}
		
		return null;
	}
	
	public String getHandle(String uri) {
		if (uri.length() == 0) {
			return "";
		}
		int pos = uri.indexOf(handleSeparator, 1);
		return pos == -1? uri : uri.substring(0, pos);
	}
	
	/**
	 * Remove the action mapping for one action class
	 * @param actionClass the action class to be removed
	 * @return the removed ActionMapping instance, null if not found
	 */
	public ActionMapping removeActionMapping(Class<? extends Action> actionClass) {
		ActionMapping mapping = mappingsByClassname.remove(actionClass.getName());
		if (mapping != null) {
			// remove from mappings by short name
			for (Iterator<Map.Entry<String, ActionMapping>> i = mappingsByUnqualifiedClassname.entrySet().iterator(); i.hasNext(); ) {
				Map.Entry<String, ActionMapping> entry = i.next();
				if (entry.getValue() == mapping) {
					i.remove();
				}
			}
			
			// remove from events
			for (Event event : mapping.getEvents().values()) {
				for (Iterator<Map.Entry<String, Event>> i = eventsByHandle.entries().iterator(); i.hasNext(); ) {
					Map.Entry<String, Event> entry = i.next();
					if (entry.getValue() == event) {
						i.remove();
					}
				}
			}
		}
		return mapping;
	}

	/**
	 * Create an {@link Action} instance for the corresponding {@link ActionMapping}
	 */
	public Action createActionInstance(ActionMapping mapping) {
		return (Action) applicationContext.getBean(mapping.getBeanName());
	}
	
	/**
	 * Validate the registered ActionMappings to check that there are no two 
	 * overlapping events that may produce the same URL 
	 */
	public void validateActionMappings() {
		// validate all actions with the same handle
		for (String key : this.eventsByHandle.keySet()) {
			List<Event> events = new ArrayList<Event>();
			events.addAll(eventsByHandle.get(key));
			for (int i = 0; i < events.size(); i++) {
				Event event = events.get(i);
				for (int j = i + 1; j < events.size(); j++) {
					event.assertNoOverlapping(events.get(j));
				}
			}
		}
	}
	
	public void addActionMapping(ActionMapping mapping) {
		String handle = mapping.getHandle();
		if (eventsByHandle.containsKey(handle) && log.isWarnEnabled()) {
			// performance is affected when the handle is shared because I have to iterate 
			// sequentially parsing each event until a match is found
			ActionMapping actual = eventsByHandle.get(handle).iterator().next().getActionMapping();
			log.warn("Multiple actions share the same handle '" + handle + "' (" +
					actual.getShortName() + ", " + mapping.getShortName() + 
					"). This is supported, but will affect performance (handles should be unique).");
		}
		eventsByHandle.putAll(handle, mapping.getEvents().values());
		mappingsByClassname.put(mapping.getLongName(), mapping);
		mappingsByUnqualifiedClassname.put(mapping.getShortName(), mapping);
		log.info("Added mapping for ", mapping.getShortName(), " at ", handle);
	}

	/**
	 * @param actionClassName the class name OR the unqualified class name (without the package path)
	 * @return the actionMapping
	 * @throws IllegalArgumentException if there is no action registered with that class name
	 */
	public ActionMapping getActionMappingByName(String actionClassName) {
		ActionMapping result = mappingsByClassname.get(actionClassName);
		if (result == null) {
			result = mappingsByUnqualifiedClassname.get(actionClassName);
		}
		if (result == null) {
			throw new ActionNotFoundException(actionClassName);
		}
		return result;
	}
	
	/**
	 * @param actionClass the action class
	 * @return the required actionMapping
	 * @throws IllegalArgumentException if there is no action registered with that class name
	 */
	public ActionMapping getActionMappingByClass(Class<? extends Action> actionClass) {
		return getActionMappingByName(actionClass.getName());
	}

	public Map<String, ActionMapping> getMappings() {
		return mappingsByClassname;
	}

	public String getHandleSeparator() {
		return handleSeparator;
	}

	public void setHandleSeparator(String handleSeparator) {
		this.handleSeparator = handleSeparator;
	}

	public void setApplicationContext(ApplicationContext applicationContext) {
		this.applicationContext = applicationContext;
	}

	public void setActionMappingFactory(ActionMappingFactory actionMappingFactory) {
		this.actionMappingFactory = actionMappingFactory;
	}

	public ApplicationContext getApplicationContext() {
		return applicationContext;
	}

	public ActionMappingFactory getActionMappingFactory() {
		return actionMappingFactory;
	}

	private static class StringComparator implements Comparator<String> {

		public int compare(String o1, String o2) {
			return o1.compareTo(o2);
		}
		
	}

}
