/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.persistence.file;

import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Transient;
import javax.persistence.Version;

import org.loom.servlet.params.FileParameter;

/**
 * Persists file info to the database, but not file contents. 
 * Used to display lists of files without retrieving current file contents.
 * @author icoloma
 */
@Entity
public class PersistentFile {

	@Id @GeneratedValue
	private Integer id;
	
	/** the filename */
	@Basic(optional = false)
	private String filename;
	
	/** the uploaded file size */
	private Long fileSize;
	
	/** the (browser-provided) content type, or guessed by the system if none was provided */
	private String contentType;

	/** the creation date, if supported. If not (e.g. the local filesystem) it will be the same as lastUpdateDate */
	private Date creationDate = new Date();
	
	/** the last update date, defaults to creationDate */
	private Date lastModified = creationDate;
	
	/** the username that persisted the file, if any */
	private String username;
	
	/** the MD5 hash of this file */
	@Column(length=32)
	private String MD5;
	
	/** the revision number */
	@Version
	private Integer version; 
	
	/** the file contents, managed without JPA */
	@Transient
	private PersistentFileContents contents;
	
	public PersistentFile() {
	}
	/**
	 * Transforms an uploaded file into a PersistentFile instance
	 */
	public PersistentFile(FileParameter value) {
		try {
			filename = value.getFilename();
			fileSize = value.getFileSize();
			contentType = value.getContentType();
			setContentsAsStream(value.getStream());
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}	
	
	public String getFilename() {
		return filename;
	}

	public Integer getId() {
		return id;
	}

	public String getContentType() {
		return contentType;
	}

	public Long getFileSize() {
		return fileSize;
	}

	public void setContentType(String contentType) {
		this.contentType = contentType;
	}

	public void setFilename(String filename) {
		this.filename = filename;
	}

	public void setFileSize(Long fileSize) {
		this.fileSize = fileSize;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public Date getCreationDate() {
		return creationDate;
	}

	public void setCreationDate(Date creationDate) {
		this.creationDate = creationDate;
		if (this.lastModified == null) {
			this.lastModified = creationDate;
		}
	}

	public PersistentFileContents getContents() {
		return contents;
	}

	public void setContents(PersistentFileContents contents) {
		this.contents = contents;
		if (fileSize  == null) {
			fileSize = contents.getFileSize();
		}
	}
	
	/**
	 * creates a new instance of PersistentFileContents
	 * @param contentsAsStream
	 */
	public void setContentsAsStream(InputStream contentsAsStream) {
		setContents(new InputStreamFileContents(contentsAsStream));
	}
	public InputStream getContentsAsStream() {
		return contents.getContentsAsStream();
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public Date getLastModified() {
		return lastModified;
	}

	public void setLastModified(Date lastModified) {
		this.lastModified = lastModified;
	}

	public String getMD5() {
		return MD5;
	}

	public void setMD5(String md5) {
		MD5 = md5;
	}
	public Integer getVersion() {
		return version;
	}
	public void setVersion(Integer version) {
		this.version = version;
	}

	
}
