package org.loom.resources;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.DigestOutputStream;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.zip.GZIPOutputStream;

import org.apache.commons.io.IOUtils;
import org.loom.exception.SyntaxError;
import org.loom.log.Log;
import org.loom.resources.compressor.WebResourceCompressor;
import org.loom.util.FileUtils;
import org.loom.util.StringUtils;

public class DiskWebResourceDataFactory extends AbstractWebResourceDataFactory {
	
	private static Log log = Log.getLog();
	
	private boolean overwriteExistingFiles = false;
	
	/**
	 * Create and register a {@link WebResourceData}
	 * @param resource the resource identifier 
	 * @return the created {@link WebResourceData}
	 * @throws IOException 
	 */
	@Override
	public WebResourceData create(WebResource resource) {
		try {
			
			File tmpFile = File.createTempFile("loom-tmp-", resource.getType().getExtension());
			
			// concatenate, minify and calculate MD5
			InputStream is = webResourceBundleRepository.getInputStream(resource);
			MessageDigest digest = MessageDigest.getInstance("MD5");
			DigestOutputStream os = null;
			try {
				os = new DigestOutputStream(new FileOutputStream(tmpFile), digest);
				WebResourceCompressor compressor = getCompressor(resource);
				compressor.compress(is, os);
			} finally {
				IOUtils.closeQuietly(is);
				IOUtils.closeQuietly(os);
			}
			
			// rename the file to MD5-based filename
			String md5 = StringUtils.toString(digest);
			String tmpDir = System.getProperty("java.io.tmpdir");
			if (tmpDir.charAt(tmpDir.length() - 1) != File.separatorChar) {
				tmpDir += File.separatorChar;
			}
			String newFileName = tmpDir + "loom-" + md5 + resource.getType().getExtension();
			log.debug("Storing WebResource [", resource.getName(), "] at ", newFileName);
			File file = new File(newFileName);
			if (file.exists() && overwriteExistingFiles) {
				file.delete();
			}
			if (!file.exists() && !tmpFile.renameTo(file)) {
				throw new RuntimeException("Could not rename " + tmpFile.getAbsolutePath() + " to " + newFileName);
			}
			
			// gzip 
			File gzippedResource = new File(newFileName + ".gz");
			FileInputStream input = null;
			GZIPOutputStream output = null;
			try {
				input = new FileInputStream(file);
				output = new GZIPOutputStream(new FileOutputStream(gzippedResource));
				FileUtils.copy(input, output);
			} finally {
				IOUtils.closeQuietly(input);
				IOUtils.closeQuietly(output);
			}

			// create the result instance
			DiskWebResourceData data = new DiskWebResourceData();
			data.setWebResource(resource);
			data.setFile(file);
			data.setGzippedFile(gzippedResource);
			data.setMd5(md5);
			data.setType(resource.getType());
			data.setCharset(resource.getCharset());
			return data;
			
		} catch (SyntaxError e) {
			throw new RuntimeException("Error processing resource: " + resource.getName(), e);
		} catch (NoSuchAlgorithmException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void setOverwriteExistingFiles(boolean overwriteExistingFiles) {
		this.overwriteExistingFiles = overwriteExistingFiles;
	}

	
}
