package org.loom.resources;

import static org.loom.util.StringUtils.calculateMD5;

import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.IOUtils;
import org.loom.exception.HttpException;
import org.loom.exception.RuntimeFileNotFoundException;
import org.loom.log.Log;
import org.loom.util.StringUtils;
import org.springframework.context.ResourceLoaderAware;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.io.support.ResourcePatternResolver;

/**
 * Default implementation of WebResourceBundle 
 * @author icoloma
 */
public class WebResourceBundleRepository implements ResourceLoaderAware {
	
	/** the list of configured WebResourceBundle, the name is key */
	private Map<String, WebResourceBundle> bundles = new HashMap<String, WebResourceBundle>();
	
	/** resolve Resource locations */
	private ResourcePatternResolver resolver;
	
	/** the encoding used for resources, default UTF-8 */
	private String charset = "UTF-8";
	
	private static final String URL_PREFIX = "url(";
	
	private static Log log = Log.getLog();
	
	/**
	 * @param name the name of the WebResourceBundle
	 * @throws IllegalArgumentException if no bundle  could be found with the given name
	 * @return the WebResourceBundle with the given name
	 */
	public WebResourceBundle getResourceBundle(String name) {
		WebResourceBundle bundle = bundles.get(name);
		if (bundle == null) {
			throw new IllegalArgumentException("WebResourceBundle with name '" + name + "' could not be found");
		}
		return bundle;
	}
	
	public IndexedWebResource createIndexedWebResource(String bundleName, int index, String filename) {
		WebResourceBundle bundle = getResourceBundle(bundleName);
		if (index >= bundle.getResources().size()) {
			throw new RuntimeFileNotFoundException("Could not find resource #" + index + " inside bundle " + bundle.getName());
		}
		
		Resource resource = getRelativeResource(bundle.getResource(index), filename);
		IndexedWebResource result = new IndexedWebResource(index, resource);
		result.setCharset(bundle.getCharset());
		return result;
	}

	/**
	 * 
	 * @param rootResource the name of the root resource
	 * @param filename the name of the resource to return, relative to the root resource
	 * @return A resource relative to the parent resource
	 */
	private Resource getRelativeResource(Resource rootResource, String filename) {
		try {
			// check for the last '/' or, if missing, the protocol
			URL url = rootResource.getURL();
			String absolutePath = url.toString();
			int lastPos = absolutePath.lastIndexOf('/');
			
			// calculate the requested resource
			return resolver.getResource(lastPos == -1? url.getProtocol() + ":" + filename : absolutePath.substring(0, lastPos + 1) + filename);
		} catch (IOException e) {
			throw new HttpException(404, e);
		}

	}

	/**
	 * Add a WebResourceBundle
	 * @param bundle
	 */
	public void addResourceBundle(WebResourceBundle bundle) {
		bundle.setCharset(charset);
		bundles.put(bundle.getName(), bundle);
	}
	/*
	public void setBundles(Map<String, Resource[]> resources) {
		for (Map.Entry<String, Resource[]> entry : resources.entrySet()) {
			String name = entry.getKey();
			log.debug("Adding resource bundle: ", name);
			WebResourceBundle bundle = new WebResourceBundle();
			bundle.setName(name);
			for (Resource resource : entry.getValue()) {
				bundle.add(resource);
			}
			addResourceBundle(bundle);
		}
		
	}*/
	
	public void setBundles(List<WebResourceBundle> b) {
		for (WebResourceBundle webResourceBundle : b) {
			addResourceBundle(webResourceBundle);
		}
	}
	
	/**
	 * Translate all relative URLs in a CSS file
	 * Only links that are relative to the CSS file location and do not contain the '../' sequence will be translated. 
	 * Absolute paths or paths pointing to other servers will not be modified.
	 * 
	 * @param resourceIndex the index of this item in the containing ResourceBundle
	 * @param resource the CSS file whose urls will be translated
	 * @throws IOException 
	 */
	public String transformCssUrls(int resourceIndex, Resource resource) {
		InputStream input = null;
		try {
			input = resource.getInputStream();
			String css = IOUtils.toString(input, charset);
			StringBuilder result = new StringBuilder(css.length());
			int i = 0;
			int pos;
			do {
				pos = css.indexOf(URL_PREFIX, i);
				if (pos == -1) {
					result.append(css, i, css.length());
				} else {
					result.append(css, i, pos);
					result.append(URL_PREFIX);
					i = pos + URL_PREFIX.length();
					while (true) {
						char c = css.charAt(i);
						if (c != '\'' && c != '"' && c != ' ') {
							break;
						}
						result.append(c);
						i++;
					}
					
					// append the md5 of the image and the index inside the bundle
					if (css.charAt(i) != '/' && !css.startsWith("http://", i) && !css.startsWith("https://", i) && !css.startsWith("../", i)) {
						int filenameEnd = StringUtils.indexOfAny(css.substring(i), new char[] { '"', '\'', ')' });
						if (filenameEnd == -1) {
							throw new IllegalArgumentException("Unterminated url() at " + resource);
						}
						String filename = css.substring(i, i + filenameEnd);
						String md5;
						try {
							md5 = calculateMD5(getRelativeResource(resource, filename));
						} catch (FileNotFoundException e) {
							// the file is not there
							log.debug("Missing resource " + filename + " referenced from CSS file " + resource.getFilename());
							md5 = "0";
						}
						result.append("../../").append(resourceIndex).append('/').append(md5).append('/');
					}
				}
			} while (pos != -1);
			return result.toString();
		} catch (IOException e) {
			throw new RuntimeException(e);
		} finally {
			IOUtils.closeQuietly(input);
		}
	}
	
	/**
	 * @return InputStream with the concatenation of all the registered resources. Translate CSS urls, if necessary
	 */
	public InputStream getInputStream(WebResource wr) throws IOException {
		List<InputStream> i = new ArrayList<InputStream>();
		int index = wr instanceof IndexedWebResource? ((IndexedWebResource)wr).getIndex() : 0;
		for (Resource resource : wr.getResources()) {
			try {
				InputStream inputStream; 
				if (WebResourceType.CSS.equals(wr.getType())) {
					String s = transformCssUrls(index, resource);
					inputStream = new ByteArrayInputStream(s.getBytes(charset));
				} else {
					inputStream = resource.getInputStream();
				}
				i.add(inputStream);
				index++;
			} catch (FileNotFoundException e) {
				throw new HttpException(400, "Could not find " + resource.getDescription());
			}
		}
		return new MultipleInputStream(i.iterator());
		
	}
	
	/**
	 * @return the list of all registered bundles
	 */
	public Collection<WebResourceBundle> getAllBundles() {
		return bundles.values();
	}
	
	public void setResolver(ResourcePatternResolver resolver) {
		this.resolver = resolver;
	}

	public void setResourceLoader(ResourceLoader resourceLoader) {
		this.resolver = (ResourcePatternResolver) resourceLoader;
	}

	public String getCharset() {
		return charset;
	}

	public void setCharset(String charset) {
		this.charset = charset;
	}

}
