package org.loom.resources;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.loom.config.ResourcesModificationListener;
import org.loom.log.Log;
import org.springframework.core.io.Resource;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.SetMultimap;

public class WebResourcesRepository implements ResourcesModificationListener {

	/** the list of cached WebResource to deliver to the browser */
	private Map<String, WebResourceData> resources = new ConcurrentHashMap<String, WebResourceData>();
	
	/** creates WebResourceData instances */
	private WebResourceDataFactory webResourceDataFactory;
	
	/** the WebResourceBundleRepository instance to resolve bundles */
	private WebResourceBundleRepository webResourceBundleRepository;
	
	/** 
	 * Lookup by Resource, for fast reloading. 
	 * The key is the physical file name, the value is the resources key.
	 */
	private SetMultimap<String, String> lookupByresource = HashMultimap.create();
	
	private static Log log = Log.getLog();
	
	/**
	 * Return a {@link WebResourceData} for the specified bundle. 
	 * @param bundleName the name of the WebResourceBundle to instantiate, process and return.
	 */
	public WebResourceData getResourceData(String bundleName) throws IOException  {
		WebResourceData res = resources.get(bundleName);
		if (res == null) {
			res = createResourceData(bundleName);
		}
		return res;
	}
	
	/**
	 * Return a {@link WebResourceData} for the specified bundle. 
	 * @param bundleName the name of the WebResourceBundle to instantiate, process and return.
	 * @param index (optional) the index of the resource to retrieve inside the bundle. If null, it will return the entire bundle.
	 */
	public WebResourceData getResourceData(String bundleName, Integer index, String filename) throws IOException {
		String key = createKey(bundleName, index,  filename);
		WebResourceData res = resources.get(key);
		if (res == null) {
			res = createResourceData(key, bundleName, index, filename);
		}
		return res;
	}
	
	private String createKey(String bundleName, Integer index, String filename) {
		return index == null? bundleName : bundleName + "/" + index + "/" + filename;
	}

	/**
	 * Create a web resource bundle 
	 */
	private synchronized WebResourceData createResourceData(String bundleName) throws IOException {
		
		WebResourceData res = resources.get(bundleName);
		if (res != null) {
			return res;
		}
		
		WebResourceData data = webResourceDataFactory.create(bundleName);
		register(bundleName, data);
		return data;
			
	}
	
	/**
	 * Create an indexed web resource (bundle + index + relative resource name)
	 * @throws IOException If there was any problem trying to read the file
	 */
	private synchronized WebResourceData createResourceData(String key, String bundleName, Integer index, String filename) throws IOException {
		// double check because we just entered a synchronized block
		WebResourceData res = resources.get(key);
		if (res != null) {
			return res;
		}
		
		WebResourceData data = webResourceDataFactory.create(bundleName, index, filename);
		register(key, data);
		return data;
			
	}
	
	private void register(String key, WebResourceData data) throws IOException {
		resources.put(key, data);
		for (Resource res : data.getWebResource().getResources()) {
			lookupByresource.put(res.getURL().getPath(), key);
		}
	}
	
	public WebResourceBundle getResourceBundle(String bundleName) {
		return webResourceBundleRepository.getResourceBundle(bundleName);
	}
	
	public void addResourceBundle(WebResourceBundle bundle) {
		webResourceBundleRepository.addResourceBundle(bundle);
	}

	public Collection<Resource> getResources() {
		List<Resource> l = new ArrayList<Resource>();
		for (WebResourceBundle bundle : webResourceBundleRepository.getAllBundles()) {
			l.addAll(bundle.getResources());
		}
		return l;
	}
	
	public void resourcesModified(List<Resource> modifiedResources) {
		try {
			for (Resource resource : modifiedResources) {
				for (String key: lookupByresource.get(resource.getURL().getPath())) {
					log.debug("Discarding cached resource '", key, "' because it has been modified");
					this.resources.remove(key);
					// there is no need to remove from lookupbyResource, as the dependency does not change (only the file contents)
				}
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void setWebResourceDataFactory( WebResourceDataFactory webResourceDataFactory) {
		this.webResourceDataFactory = webResourceDataFactory;
	}

	public void setDebug(boolean development) {
		this.webResourceDataFactory.setDebug(development);
	}

	public void setWebResourceBundleRepository(
			WebResourceBundleRepository webResourceBundleRepository) {
		this.webResourceBundleRepository = webResourceBundleRepository;
	}

}
