/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import org.apache.commons.lang.StringUtils;

/**
 * Contains the browser data that can be guessed from the User-Agent header.
 * Keep in mind that the user can modify the User-Agent string, so BrowserData 
 * is not 100% bulletproof. To be sure you should be using javascript instead.
 * 
 * For examples of User-Agent header values, see 
 * http://www.zytrax.com/tech/web/browser_ids.htm
 * http://www.user-agents.org/index.shtml
 * @author icoloma
 */
public class BrowserData {

	private static final String IE_PREFIX = "Mozilla/4.0 (compatible; MSIE ";
	
	/** the user-agent header contents */
	private String userAgent;

	/** true if the browser is gecko-compliant (lazily initialized) */
	private Boolean gecko;
	
	/** true if the browser is IE (lazily initialized) */
	private Boolean IE;
	
	/** the version number, null if not yet calculated */
	private String version;
	
	public BrowserData(String userAgent) {
		this.userAgent = userAgent;
	}
	
	/**
	 * @return true if the browser is a IE
	 */
	public boolean isIE() {
		if (IE == null) {
			IE = userAgent != null && userAgent.startsWith(IE_PREFIX);
		}
		return IE;
	}
	
	/**
	 * @return true if the browser is Firefox or any other Gecko-based browser
	 */
	public boolean isGecko() {
		if (gecko == null) {
			gecko = userAgent != null && userAgent.indexOf("Gecko") != -1 && userAgent.indexOf("KHTML") == -1;
		}
		return gecko;
	}
	
	/**
	 * @return the version number if it could be extracted, else return an empty String.
	 */
	public String getVersion() {
		
		if (version == null) {
			if (userAgent != null) {
				if (isIE()) {
					version = userAgent.substring(IE_PREFIX.length(), userAgent.indexOf(';', IE_PREFIX.length()));
				} else if (isGecko()) {
					version = StringUtils.substringAfterLast(userAgent, "/");
				} 
				// TODO add safari and opera
			}
			if (version == null) {
				version = "";
			}
		}
		return version;
	}
	
	/**
	 * @return true if the browser version is less than the provided version, or false if the version number could not be guessed.
	 */
	public boolean isVersionLessThan(String version) {
		String actual = getVersion();
		return actual.length() == 0? false : version.compareTo(actual) > 0;
	}

	public String getUserAgent() {
		return userAgent;
	}
	
}
