package org.loom.servlet;


import static org.loom.servlet.names.CommonHeaderNames.CACHE_CONTROL;
import static org.loom.servlet.names.RequestHeaderNames.IF_MODIFIED_SINCE;
import static org.loom.servlet.names.RequestHeaderNames.IF_NONE_MATCH;
import static org.loom.servlet.names.ResponseHeaderNames.ETAG;
import static org.loom.servlet.names.ResponseHeaderNames.EXPIRES;
import static org.loom.servlet.names.ResponseHeaderNames.LAST_MODIFIED;

import java.util.Date;

import org.loom.log.Log;
import org.loom.servlet.names.ResponseHeaderNames;

/**
 * Contains the caching information related to the response object.
 * This information tells the browser about how it is expected to 
 * cache the server response. 
 * 
 * @author icoloma
 */
public class CacheControl {
	
	/** the last modification date, to be used as Last-Modified header */
	private Date lastModified;
	
	/** the ETag value */
	private String etag;
	
	/** the number of seconds to be stored in the cache */
	private Integer cacheSeconds;
	
	/** true to never cache */
	private Boolean cacheNever;

	private static Log log = Log.getLog();
	
	/**
	 * Compares this CacheControl data with the request headers, and returns true if the browser page 
	 * is outdated and a new one should be sent. If false, a 304 could be sent instead.
	 * If both lastModified and etag has been specified, this method will return true if any 
	 * is missing or incorrect in the browser request.
	 * @param request the current servlet request
	 * @return true if a new page should be sent, false to send a 304
	 */
	public boolean isResourceModified(LoomServletRequest request) {
		if (Boolean.TRUE.equals(cacheNever) || ( lastModified == null && etag == null && cacheSeconds == null)) {
			log.debug(request.getRequestURI(), " is set to cacheNever or does not have set Last-Modified, ETag or cacheSeconds");
			return true;
		}
		
		String ifNoneMatch = request.getHeader(IF_NONE_MATCH);
		long ifModifiedSince = request.getDateHeader(IF_MODIFIED_SINCE);
		
		if (etag != null && !etag.equals(ifNoneMatch) ) {
			log.debug(request.getRequestURI(), " ETag does not match (server=", etag, ", browser=", ifNoneMatch, ")");
			return true;
		} else if (etag == null && ifNoneMatch != null) {
			log.warn("The request for resource '", request.getRequestURI(), "' includes a 'If-None-Match: " + 
					ifNoneMatch + "' header, but the response does not have any ETag header set. This is most probably an error.");
		}
		
		if (lastModified != null && lastModified.getTime() != ifModifiedSince) {
			log.debug(request.getRequestURI(), " Last-Modified does not match (server=", lastModified.getTime(), ", browser=", ifModifiedSince, ")");
			return true;
		} 
		
		return false;
	}
	
	/**
	 * Write the headers for a 200 (OK) or 304 (NOT_MODIFIED) response. 
	 */
	public void writeHeaders(LoomServletResponse response) {
		if (lastModified != null) {
			response.setDateHeader(LAST_MODIFIED, lastModified.getTime());
		}
		if (etag != null) {
			response.setHeader(ETAG, etag);
		}
		if (cacheSeconds != null) {
			response.setDateHeader(EXPIRES, System.currentTimeMillis() + cacheSeconds * 1000L);
			response.setHeader(CACHE_CONTROL, "max-age=" + cacheSeconds);
		}
		if (Boolean.TRUE.equals(cacheNever)) {
			response.setDateHeader(EXPIRES, 1L);
			response.setHeader(ResponseHeaderNames.PRAGMA, "no-cache"); 
			
			// firefox uses "no-store", everyone else uses "no-cache"
			response.setHeader(CACHE_CONTROL, "no-cache, no-store, must-revalidate"); 
		}
	}

	/**
	 * Sets the response to never expire.
	 * Invoking this method will use Steven Souders recommendation of setting the cache to ten years in the future, 
	 * instead of the HTTP spec recommendation of one year.
	 */
	public void setCacheForever() {
		setCacheYears(10);
	}

	public Date getLastModified() {
		return lastModified;
	}

	public void setLastModified(Date lastModified) {
		// the browser-sent header will only contain seconds and no millis info, 
		// so round up the millis to match its value
		this.lastModified = new Date(1000 * (lastModified.getTime() / 1000));
	}

	public String getEtag() {
		return etag;
	}

	public void setEtag(String etag) {
		this.etag = etag;
	}

	public Integer getCacheSeconds() {
		return cacheSeconds;
	}

	/**
	 * @param seconds the number of seconds to store the resource in the browser cache, starting now
	 */
	public void setCacheSeconds(Integer seconds) {
		this.cacheSeconds = seconds;
	}
	
	/**
	 * @param minutes the number of minutes to store the resource in the browser cache, starting now
	 */
	public void setCacheMinutes(int minutes) {
		setCacheSeconds(minutes * 60);
	}
	
	/**
	 * @param hours the number of hours to store the resource in the browser cache, starting now
	 */
	public void setCacheHours(int hours) {
		setCacheSeconds(hours * 60 * 60);
	}
	
	/**
	 * @param days the number of days to store the resource in the browser cache, starting now
	 */
	public void setCacheDays(int days) {
		setCacheSeconds(days * 24 * 60 * 60);
	}
	
	/**
	 * @param years the number of years to store the resource in the browser cache, starting now
	 */
	public void setCacheYears(int years) {
		setCacheSeconds(years * 365 * 24 * 60 * 60);
	}

	public Boolean getCacheNever() {
		return cacheNever;
	}

	public void setCacheNever() {
		this.cacheNever = true;
	}
	
}
