/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;

import org.loom.action.Action;
import org.loom.i18n.MessagesRepository;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.params.FileParameter;
import org.loom.servlet.params.ServletRequestParameters;
import org.loom.tags.core.FormTag;
import org.loom.url.UrlBuilder;

/**
 * An enhanced implementation of the HttpServletRequest interface
 * @author icoloma
 *
 */
public interface LoomServletRequest extends HttpServletRequest {

	/**
	 * @return the MessagesRepository corresponding to the this request
	 */
	public MessagesRepository getMessagesRepository();

	/**
	 * @return the action that is handling this request,  null if none
	 */
	public ParsedAction getParsedAction();
	
	/**
	 * TODO: this method should be moved elsewhere
	 * @return the current JSP form tag, null if none.
	 */
	public FormTag getCurrentForm();
	public void setCurrentForm(FormTag currentForm);

	/**
	 * Generates a valid HTML id starting with prefix. 
	 * @param prefix the prefix to use to generate the id
	 * @return A generated correlative id starting at 0: prefix_0, prefix_1, etc.
	 */
	public String generateId(String prefix);
	
	/**
	 * Generate a correlative index for the field passed as argument, in the form: field[0], field[1], etc.
	 * To generate HTML ID for an entire web page, see generateId
	 * @return the generated field name, e.g.: "field[0]"
	 */
	public String generateIndex(String name);

	/**
	 * Add an uploaded file parameter
	 * @param name the file parameter name
	 * @param item the uploaded file to add to this request
	 */
	public void addFileParameter(String name, FileParameter item);
	
	/**
	 * Add a String parameter
	 * @param name the name of the parameter  
	 * @param value the value of the parameter 
	 */
	public void addStringParameter(String name, String value);
	
	/**
	 * @return a {@link ServletRequestParameters} instance with all file upload parameters and 
	 * plain parameters in a single map.
	 */
	public ServletRequestParameters getRequestParameters();
	
	/**
	 * @return the GET/POST parameter value
	 */
	public String getStringParameter(String name);
	
	/**
	 * @return the uploaded file
	 */
	public FileParameter getFileParameter(String name);
	
	/**
	 * @param cookieName the name of the cookie
	 * @return the cookie value, null if none
	 */
	public Cookie getCookie(String cookieName);

	/**
	 * Returns the full url including query parameters, protocol, server and port.
	 * This will convert a POST request into a GET one, if there are no uploaded files.
	 * @throws RuntimeException if the request include any uploaded files.
	 * @deprecated use toUrlBuilder().getURL() instead
	 */
	public String getRequestUrlWithParameters();

	/**
	 * Return true if the current request has been launched using Ajax.
	 * Actually, it looks for a 'X-Requested-With' HTTP header, that is added by JQuery and prototype.
	 * @return true if the request has been launched using XmlHttpRequest, false if this is a normal request.
	 */
	public boolean isAjax();
	
	/**
	 * @return a request attribute, as a boolean. If null, returns false.
	 */
	public boolean getAttributeAsBoolean(String attributeName);

	/**
	 * @return the data that can be sniffed from the browser.  
	 */
	public BrowserData getBrowserData();

	/**
	 * Create a scoped atribute
	 * @param name the attribute name
	 * @param value the value 
	 * @param scope the scope of the attribute. Scope.PAGE is not allowed. 
	 */
	public void setScopedAttribute(String name, Object value, Scope scope);

	/**
	 * Get a scoped attribute
	 * @param name the attribute name
	 * @param scopes the list of scopes to inspect, null for all. Scope.PAGE is not allowed.
	 */
	public Object getScopedAttribute(String name, Scope... scopes);
	
	/**
	 * Remove a scoped atribute
	 * @param name the attribute name
	 * @param scope the scope of the attribute. Scope.PAGE is not allowed. 
	 */
	public void removeScopedAttribute(String name, Scope scope);
	
	/**
	 * @return the HttpMethod of this request. Note that the method used by the browser could be overriden 
	 * if this is a POST request and the _method parameter is present. 
	 */
	public HttpMethod getHttpMethod();

	/**
	 * Create a {@link UrlBuilder} instance with the same action/event/rawUrl and all the parameters 
	 * included in this request instance. Any file parameters will be lost.
	 * @return the craeted {@link UrlBuilder} instance.
	 */
	public UrlBuilder toUrlBuilder();

	/**
	 * Creates a new empty {@link UrlBuilder}
	 */
	public UrlBuilder createUrl();
	
	/**
	 * Creates a new {@link UrlBuilder} pointing to the provided action and event 
	 */
	public UrlBuilder createUrl(Class<? extends Action> actionClass, String event);
	
	/**
	 * Creates a new {@link UrlBuilder} pointing to the provided raw url 
	 */
	public UrlBuilder createUrl(String url);
	
}