/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.LinkedList;
import java.util.List;

import javax.servlet.ServletOutputStream;
import javax.servlet.ServletResponse;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

import org.loom.config.Config;


/**
 * Wrap the real response object.
 * @author icoloma
 */
public class LoomServletResponseImpl extends HttpServletResponseWrapper implements LoomServletResponse {
	
	/** true if the start event has been fired */
	private boolean startFired;
	
	/** the list of event listeners */
	private List<ResponseListener> listeners = new LinkedList<ResponseListener>();
	
	/** the assigned status code, if any. Defaults to 200 (OK) */
	private int status = SC_OK;
	
	/**
	 * Returns the instance of LoomServletResponse if one already exists, or creates a new one if not
	 */
    public static LoomServletResponseImpl getInstance(LoomServletRequest request, HttpServletResponse parent) {
    	LoomServletResponseImpl response = internalGetInstance(parent);
    	if (response != null) {
    		return response;
    	}

        // set the same Locale that has been calculated for the request object
        response = new LoomServletResponseImpl(parent);
        response.setLocale(request.getLocale());
		
		// set the default response encoding
		String defaultCharset = Config.getInstance().getDefaultCharset();
		if (defaultCharset != null) {
			response.setCharacterEncoding(defaultCharset);
		}

        return response;
    }
    
    /**
     * Returns the instance of LoomServletResponse if one already exists, throw exception if not
     */
    public static LoomServletResponseImpl getInstance(ServletResponse parent) {
    	LoomServletResponseImpl response = internalGetInstance(parent);
    	if (response == null) 
    		throw new IllegalArgumentException("Instance of LoomServletResponse could not be found");
    	return response;
    }
    
    
    private static LoomServletResponseImpl internalGetInstance(ServletResponse response) {
    	while ( response != null && response instanceof HttpServletResponseWrapper) {
    		if (response instanceof LoomServletResponseImpl)
    			return (LoomServletResponseImpl) response;
    		response = ((HttpServletResponseWrapper) response).getResponse();
    	}
    	// not found
    	return null;
    }
	
    private LoomServletResponseImpl(HttpServletResponse nested) {
    	super(nested);
    }

	public void deleteCookie(String name) {
		Cookie c = new Cookie(name, "deleted");
	    c.setMaxAge(0); 
	    c.setPath("/"); 
	    super.addCookie(c);
	}
	
	/**
	 * The addCookie behaviour has been changed. By default cookies are asociated to the webapp context,
	 * since it is the common case.
	 */
	@Override
	public void addCookie(Cookie cookie) {
		if (cookie.getPath() == null) {
			cookie.setPath("/");
		}
		super.addCookie(cookie);
	}

	@Override
	public ServletOutputStream getOutputStream() throws IOException {
		fireResponseStart();
		return super.getOutputStream();
	}
	
	@Override
	public PrintWriter getWriter() throws IOException {
		fireResponseStart();
		return super.getWriter();
	}

	private void fireResponseStart() {
		if (!startFired) {
			startFired = true;
			for (ResponseListener listener : listeners) {
				listener.responseStart(this);
			}
		}
	}

	public void addListener(ResponseListener listener) {
		this.listeners.add(listener);
	}

	@Override
	public void setStatus(int sc) {
		super.setStatus(sc);
		this.status = sc;
	}
	
	@Override
	public void sendError(int sc) throws IOException {
		super.sendError(sc);
		this.status = sc;
	}
	
	@Override
	public void sendError(int sc, String msg) throws IOException {
		super.sendError(sc, msg);
		this.status = sc;
	}
	
	public int getStatus() {
		return status;
	}
	
}
