/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.servlet.params;

import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.fileupload.FileItemIterator;
import org.apache.commons.fileupload.FileItemStream;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.builder.ToStringBuilder;
import org.loom.config.Config;
import org.loom.servlet.FlashContainer;
import org.loom.servlet.names.FlashAttributeNames;

import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

/**
 * Parameters as received by the request object.
 * This class can only store one value per name. If more than one parameter is added
 * with the same name, only the first one will be processed. To handle multiple values,
 * use indexes (values[2].name) where needed to ensure uniqueness.
 * @author Ignacio Coloma
 */
public class ServletRequestParameters {

	/** String parameter values, uploaded via GET or POST, normal or url-form-encoded */
	private Map<String, String> stringParameters = Maps.newLinkedHashMap();

	/** uploaded file parameters, if any */
	private Map<String, FileParameter> fileParameters = Maps.newLinkedHashMap();
	
	/** set of parameters that have already been assigned */
	private Set<String> assignedParameters = Sets.newHashSet();
	
	public ServletRequestParameters() {
	}
	
	@SuppressWarnings("unchecked")
	public ServletRequestParameters(HttpServletRequest request) {
		if (ServletFileUpload.isMultipartContent(request)) {
			convertMultipartFormParameters(request);
		}
		
		// add normal parameters. If multipart, this will only add any extra GET parameter (not POST)
		for (Enumeration<String> e = request.getParameterNames(); e.hasMoreElements(); ) {
			String paramName = e.nextElement();
			addStringParameter(paramName, request.getParameter(paramName));
		}
		
		// restore old parameters from a redirect
		restoreFromRedirect(request);
		
	}
	
	/**
	 * Separate multipart form params by type: normal form parameters are
	 * returned as a Map, and the rest is stored in a list of FileItem
	 */
	private void convertMultipartFormParameters(HttpServletRequest request) {
		try {
			ServletFileUpload fileUpload = new ServletFileUpload();
			FileParameterFactory fileParameterFactory = Config.getInstance().getFileParameterFactory();

			// commons-fileupload uses the http default (ISO-8859-1) when the charset is missing, while the browser uses the same 
			// charset of the last response (which should be config.defaultCharset)
			String charset = request.getCharacterEncoding() == null? Config.getInstance().getDefaultCharset() : request.getCharacterEncoding();
			
			int index = 0;
			for (FileItemIterator iterator = fileUpload.getItemIterator(request); iterator.hasNext(); index++) {
				FileItemStream item = iterator.next();
				String name = item.getFieldName();
				if (!contains(name)) { // avoid overwriting with the last value
					if (item.isFormField()) { // simple field, add as String
						InputStream is = item.openStream();
						try {
							String value = IOUtils.toString(is, charset);
							addStringParameter(name, value);
						} finally {
							IOUtils.closeQuietly(is);
						}
					} else { 
						addFileParameter(name, fileParameterFactory.create(item));
					}
				}

			}
			
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		} catch (FileUploadException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	
	/**
	 * Adds an uploaded file item to the list of parameters
	 * @param file
	 */
	public void addFileParameter(String name, FileParameter file) {
		if (file.getFileSize() == 0) {
			return;
		}
		fileParameters.put(name, file);
	}
	
	/**
	 * Add a parameter value to the list of processed params
	 */
	public void addStringParameter(String paramName, String value) {
		stringParameters.put(paramName, value);
	}
	
	/**
	 * Adds all the provided params to this RequestParams instance
	 * @param params the Map of params to be added
	 */
	public void addStringParameters(Map<String, String> params) {
		this.stringParameters.putAll(params);
	}
	
	/**
	 * Adds all the provided params to this RequestParams instance
	 * @param params the Map of params to be added
	 */
	public void addFileParameters(Map<String, FileParameter> params) {
		this.fileParameters.putAll(params);
	}
	
	public Set<Map.Entry<String, String>> getStringParametersEntrySet() {
		return stringParameters.entrySet();
	}
	
	public Set<Map.Entry<String, FileParameter>> getFileParametersEntrySet() {
		return fileParameters.entrySet();
	}
	
	public String getStringParameter(String name) {
		return stringParameters.get(name);
	}
	
	public FileParameter getFileParameter(String name) {
		return fileParameters.get(name);
	}

	public boolean contains(String paramName) {
		return stringParameters.containsKey(paramName) || fileParameters.containsKey(paramName);
	}
	
	public int size() {
		return stringParameters.size() + fileParameters.size();
	}
	
	/**
	 * @return a list of request params in the form name=value
	 */
	@Override
	public String toString() {
		return new ToStringBuilder(getClass())
			.append("stringParameters", stringParameters)
			.append("fileParameters", fileParameters.keySet())
			.append("assignedParameters", assignedParameters).toString();
	}

	public Set<String> getStringParameterNames() {
		return stringParameters.keySet();
	}
	
	public Set<String> getFileParameterNames() {
		return fileParameters.keySet();
	}

	/**
	 * Set the provided parameter name  as assigned
	 */
	public void setAssigned(String name) {
		assignedParameters.add(name);
	}

	/**
	 * Return true if the provided parameter has already been assigned to a java counterpart (action attribute or event parameter)
	 */
	public boolean isAssigned(String paramName) {
		return assignedParameters.contains(paramName);
	}

	/**
	 * Save the string parameters in this object to the flash context, to be retored at the next request.
	 */
	public void saveForRedirect(HttpServletRequest request) {
		if (!stringParameters.isEmpty()) {
			FlashContainer flash = FlashContainer.getNext(request, true);
			flash.setAttribute(FlashAttributeNames.SAVED_PARAMETERS, stringParameters);
		}
	}

	/**
	 * restores all parameters from a previous request (after a redirect).
	 * This method is intended to restore parameters after an unsuccessful event execution because of errors in the user input data.
	 * @param request
	 */
	@SuppressWarnings("unchecked")
	public void restoreFromRedirect(HttpServletRequest request) {
		FlashContainer flash = FlashContainer.getPrev(request);
		if (flash != null) {
			Map<String, String> restoredParams = (Map<String, String>) flash.getAttribute(FlashAttributeNames.SAVED_PARAMETERS);
			if (restoredParams != null) {
				stringParameters.putAll(restoredParams);
			}
		}
	}

}
