/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags;

import java.io.IOException;
import java.io.StringWriter;
import java.io.Writer;

/**
 * A copy of java.io.StringWriter substituting StringBuffer (synchronized) with 
 * StringBuilder (not synchronized).
 *  
 * @see StringWriter
 * @author icoloma
 */
public class StringBuilderWriter extends Writer {

    private StringBuilder buf;

    /**
     * Create a new string writer using the default initial string-buffer
     * size.
     */
    public StringBuilderWriter() {
	buf = new StringBuilder();
	lock = buf;
    }

    public StringBuilderWriter(int initialSize) {
	if (initialSize < 0) {
	    throw new IllegalArgumentException("Negative buffer size");
	}
	buf = new StringBuilder(initialSize);
	lock = buf;
    }
    
    public int size() {
    	return buf.length();
    }

    public void write(int c) {
	buf.append((char) c);
    }

    public void write(char cbuf[], int off, int len) {
        if ((off < 0) || (off > cbuf.length) || (len < 0) ||
            ((off + len) > cbuf.length) || ((off + len) < 0)) {
            throw new IndexOutOfBoundsException();
        } else if (len == 0) {
            return;
        }
        buf.append(cbuf, off, len);
    }

    public void write(String str) {
	buf.append(str);
    }

    public void write(String str, int off, int len)  {
	buf.append(str.substring(off, off + len));
    }

    public StringBuilderWriter append(CharSequence csq) {
	if (csq == null)
	    write("null");
	else
	    write(csq.toString());
    	return this;
    }

    public StringBuilderWriter append(CharSequence csq, int start, int end) {
	CharSequence cs = (csq == null ? "null" : csq);
	write(cs.subSequence(start, end).toString());
    	return this;
    }

    public StringBuilderWriter append(char c) {
	write(c);
	return this;
    }

    
    public String toString() {
	return buf.toString();
    }

    
    public StringBuilder getBuilder() {
	return buf;
    }

    
    public void flush() { 
    }

    
    public void close() throws IOException {
    }

}
