/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.loom.binding.PropertyBinder;
import org.loom.binding.PropertyWrapper;
import org.loom.binding.PropertyWrapperFactory;
import org.loom.config.Config;
import org.loom.config.ConfigProperties;
import org.loom.converter.Converter;
import org.loom.converter.StringConverter;
import org.loom.log.Log;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.tags.AbstractFormInputTag;
import org.loom.util.ClassUtils;
import org.tldgen.annotations.Attribute;

/**
 * Select or Radio field with nested options.
 * There are three ways of specifying nested options: 
 * <ul><li>Enumeration properties get their options lists automatically filled (though it can be overriden)</li>
 * <li>They can be specified using the options attribute, which must be an Iterable with either
 * {@link Option} instances or any other class in which case the labelProperty and keyProperty will be 
 * used to convert to Option objects</li>
 * <li>Options can be defined as nested l: option tags. This can be combined with any of the other two methods</li>
 * </ul>
 * 
 * @author icoloma
 */
public abstract class AbstractOptionsTag extends AbstractFormInputTag {
	
	private static Converter DEFAULT_CONVERTER = new StringConverter();
	
	public static enum SortBy {
		NONE,
		LABEL,
		VALUE
	}

	private Iterable<?> optionsAttribute;

	/** the property of each <code>options</code> value that will be used as label, default 'name' */
	@Attribute
	private String labelProperty = "name";
	
	/** the property of each <code>options</code> value that will be used as key, default 'id' */
	@Attribute
	private String keyProperty = "id";
	
	/** true to translate option labels, default true */
	@Attribute
	private boolean translateLabels = true;

	/** the Comparator class used to sort the options, null for none, default to LabelComparator */
	@Attribute
	private Class<? extends Comparator<?>> optionsComparatorClass;
	
	/** 
	 * The list of options to show inside this tag.
	 * This attribute is calculated. It includes the contents of the optionsAttribute (if specified) and any 
	 * nested {@link OptionTag}
	 */
	private List<Option> options = new ArrayList<Option>();
	
	/** true if empty lists are ok, default false */
	private static boolean EMPTY_LISTS_OK = ConfigProperties.getInstance().getPropertyAsBoolean("formInputField.emptyListsAreOk", false);
	
	/** specifies the sorting method to apply. defaults to <code>byLabel</code> */
	private SortBy sortBy = SortBy.LABEL;
	
	/** the converter to transform between text and java values */
	private Converter converter;
	
	private static Log log = Log.getLog();

	/**
	 * Returns the available options. Supports any combination of:
	 * <ul>
	 * <li>An options attribute, pointing to an Enum class</li>
	 * <li>An options attribute, listing its values separated by commas</li>
	 * <li>Nested &lt;option> tags, those will be used</li>
	 * <li>If the target action property is an enumeration, the possible values
	 * will be used</li>
	 * </ul>
	 */
	@SuppressWarnings("unchecked")
	public void calculateOptions() throws JspException, IOException {
		
		// doBody() is called just to process any nested <option> tags
		// and add them to this instance
		// this body HTML output, if any, will be ignored
		doBufferedBody();

		boolean isEnum = false;
		if (optionsAttribute != null) {
			pullOptionsFromAttribute();
		} else {
			Event event = getForm().getTargetEvent();
			Class<?> propertyClass = event.getPropertyClass(getNormalizedName());
			isEnum = propertyClass.isEnum();
			if (isEnum) {
				guessOptionsFromEnumeration((Class<? extends Enum<?>>) propertyClass);
			}
		}
		
		if (options.isEmpty()) {
			// the common case is that an empty list is a developer mistake
			if (!EMPTY_LISTS_OK) {
				throw new IllegalArgumentException( "No 'options' attribute has been specified or the list is empty for inputSelect with name = " + getName());
			}
			
			// if the list is empty, add a null option 
			options.add(Option.NULL_OPTION);
		}
		
		// default order for fields that are not Enumeration
		if (optionsComparatorClass ==  null && !isEnum) {
			optionsComparatorClass = sortBy == SortBy.LABEL? LabelComparator.class :
				sortBy == SortBy.VALUE? ValueComparator.class :
					null;
		}
		
		// if a comparator has been specified, sort the options
		if (optionsComparatorClass !=  null) {
			Comparator comparator = ClassUtils.newInstance(optionsComparatorClass);
			if (comparator instanceof I18nComparator) {
				((I18nComparator)comparator).setRepository(repository);
			}
			Collections.sort(options, comparator);
		}

	}
	
	/**
	 * @return the {@link Converter} to be used for converting each option to text
	 */
	protected Converter getConverter() {
		if (converter == null) {
			ParsedAction parsedAction = request.getParsedAction();
			if (parsedAction != null) {
				PropertyBinder propertyBinder = parsedAction.getActionMapping().getPropertyBinderMap().getPropertyBinder(getNormalizedName());
				if (propertyBinder != null) {
					converter = propertyBinder.getConverter();
				}
			} 
			if (converter == null) {
				// TODO: how to proceed here? 
				converter = DEFAULT_CONVERTER;
			}
		} 
		return converter;
	}
	
	/**
	 * If the options value is a list of Option instances, adds them 
	 */
	@SuppressWarnings("unchecked")
	public void pullOptionsFromAttribute() {
		PropertyWrapper keyPropertyWrapper = null;
		PropertyWrapper namePropertyWrapper = null;
		
		for (Iterator i = optionsAttribute.iterator(); i.hasNext(); ) {
			Object instance = i.next();
			if (instance instanceof Option) {
				// returned a Iterable<Option>
				options.add((Option)instance);
			} else {
				// returned a list of anything else, convert to Option
				if (keyPropertyWrapper == null) {
					PropertyWrapperFactory factory = Config.getInstance().getPropertyWrapperFactory();
					keyPropertyWrapper = factory.createInstance(instance.getClass(), keyProperty);
					namePropertyWrapper = factory.createInstance(instance.getClass(), labelProperty);
				}
				Comparable key = (Comparable) keyPropertyWrapper.get(instance, false);
				Object label = namePropertyWrapper.get(instance, false);
				options.add(new Option(key, label == null? null : label.toString(), translateLabels));
			}
		}
	}

	/**
	 * If the field is an Enum, translate the possible values to a list of
	 * options
	 */
	protected void guessOptionsFromEnumeration(Class<? extends Enum<?>> enumerationClass) {
		try {
			Method m = enumerationClass.getMethod("values");
			Enum<?>[] values = (Enum<?>[]) m.invoke(null);
			for (Enum<?> e : values) {
				options.add(new Option(e, e.toString(), translateLabels));
			}
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e.getTargetException());
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
	}

	/**
	 * Returns the label of the currently selected option, translated if
	 * necessary
	 */
	@Override
	public String renderAsText() {
		try {
			String result = null;
			calculateOptions();
			Object currentValue = getValueAsObject();
			if (currentValue != null) {
				for (Option option : options) {
					if (option.isSelected(currentValue)) {
						result = option.getLabel(repository);
						break;
					}
				}
				if (result == null) {
					log.error("Could not find value " + currentValue + " in the list of available options for " + getName());
					result = currentValue == null? "" : currentValue.toString();
				}
			}
			return result;
		} catch (JspException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	public void addOption(String value, String label, Boolean translateLabel) {
		boolean translate = translateLabel != null? translateLabel : isTranslateLabels();
		if (label == null) {
			label = value == null? "" : value;
		}
		Object valueAsObject = StringUtils.isEmpty(value)? null : getConverter().getAsObject(getName(), value, request.getParsedAction().getMessages(), getMessagesRepository());
		addOption(new Option(valueAsObject, label, translate));
	}
	
	public void addOption(Option option) {
		options.add(option);
	}

	public void setTranslateLabels(boolean translateLabels) {
		this.translateLabels = translateLabels;
	}

	/** 
	 * An Iterable with the available option values. If the Iterable contents are instances of 
	 * Option, they will be used as is. Otherwise, labelProperty and keyProperty will be used
	 * to retrieve the labeland key properties, respectively. This attribute can be combined
	 * with nested &lt;option> tags  
	 */
	@Attribute
	public void setOptions(Iterable<?> options) {
		this.optionsAttribute = options;
	}

	public List<Option> getNestedOptions() {
		return options;
	}

	public boolean isTranslateLabels() {
		return translateLabels;
	}

	public void setLabelProperty(String labelProperty) {
		this.labelProperty = labelProperty;
	}

	public void setKeyProperty(String keyProperty) {
		this.keyProperty = keyProperty;
	}

	@SuppressWarnings("unchecked")
	public void setOptionsComparatorClass(String comparatorClass) {
		this.optionsComparatorClass = StringUtils.isEmpty(comparatorClass)? null : (Class<Comparator<Option>>)ClassUtils.forName(comparatorClass);
	}

	/** 
	 * The method to apply for sorting options. Possible values are "none", "value" and "label" (default) 
	 */
	@Attribute
	public void setSortBy(String sortBy) {
		this.sortBy = SortBy.valueOf(sortBy.toUpperCase());
	}

	public void setConverter(Converter converter) {
		this.converter = converter;
	}


}
