/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.action.support.ResourcesAction;
import org.loom.config.Config;
import org.loom.resources.WebResourceBundle;
import org.loom.resources.WebResourceData;
import org.loom.resources.WebResourcesRepository;
import org.loom.tags.AbstractTag;
import org.tldgen.annotations.Attribute;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.springframework.core.io.Resource;

/**
 * Tag that creates a link to a script or css file.
 * There are two common uses for this tag:
 * &lt;l:script src="/js/myfile.js"/>
 * This tag will create a normal script tag, pointing to the specified resource with the webapp prefix prepended.
 *  
 * &lt;l:script resource="resource_path"/>
 * When specifying the resource, it will use a resource bundle configured using spring.
 * 
 * @author icoloma
 */
public abstract class AbstractWebResourceTag extends AbstractTag implements ParameterContainer {

	/** include the library with any version of IE */
	public static final Integer IE_ANY = -1;

	/** 
	 * If specified, the script will be surrounded by conditional Internet Explorer comments 
	 * &lt;--[if lt IE #]> ... &lt;![endif]--&gt; 
	 * Set to a number to apply to versions strictly less than specified, or "any" for any version of IE.
	 */
	@Attribute
	private Integer ieLessThan;
	
	private UrlBuilder urlBuilder = new UrlBuilder();
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/struct/links.html#adef-type-A">Link HTML attribute</a> or 
	 * <a href="http://www.w3.org/TR/html4/interact/scripts.html#adef-type-SCRIPT">script HTML attribute</a> 
	 * If unspecified, defaults to "text/css" or "text/javascript", respectively
	 */
	@Attribute
	private String type;

	/** WebResourcesRepository that this tag will use */
	private WebResourcesRepository webResourcesRepository = Config.getInstance().getWebResourcesRepository();
	
	/** 
	 * The web resource bundle to use. If not null, the specified web resource will be delivered:
	 * <ul>
	 * <li>Config.development is true: each file specified in this resource will be retrieved 
	 * separately, and no compression will take place</li>
	 * <li>Config.development is false: all files in this resource will be concatenated, minified 
	 * and gzipped in a single file before delivering to the browser</li>
	 * </ul> 
	 */
	@Attribute
	private String resource;
	
	/** true to activate development mode */
	@Attribute
	private boolean development = Config.getInstance().isDevelopment();
	
	/** the body of this tag, if any */
	private String contents;
	
	@Override
	protected void initTag() {
		try {
			super.initTag();
			
			// IE conditional comments
			if (ieLessThan != null) {
				if (IE_ANY.equals(ieLessThan)) {
					out.print("<!--[if IE]>");
				} else {
					out.print("<!--[if lt IE ").print(ieLessThan.toString()).print("]>");
				}
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		
		// register any nested <l:param>
		contents = doBufferedBody();
		
		// if bundle has been set, calculate the url
		String bundle = getResource();
		if (bundle != null) {
			if (urlBuilder.isEmpty()) { // unspecified action class, use default
				urlBuilder.setAction(ResourcesAction.class);
				urlBuilder.setEvent("getResource");
			}
			if (isDevelopment()) {
				// print one link to ResourcesAction per linked resource
				WebResourceBundle b = webResourcesRepository.getResourceBundle(bundle);
				int i = 0;
				for (Resource location : b.getResources()) {
					printResourceTag(bundle, i++, location.getFilename());
					out.print("\n");
				}
			}  else {
				// print a single link to ResourcesAction
				printResourceTag(bundle, null, null);
			}
		} else {
			// print a simple script tag
			printTag();
		}

	}
	
	/** print just the tag */
	protected abstract void printTag() throws JspException, IOException;

	/**
	 * Print a script tag pointing to a bundled resource
	 * @param bundle the name of the WebResourceBundle
	 * @param index the index inside the bundle. If null, the entire bundle will be returned
	 * @param browserFileName the name of the file. This field is not used by the server, but is added 
	 * to the URL so that the user can identify the script in the browser 
	 * 
	 */
	private void printResourceTag(String bundle, Integer index, String browserFileName) throws JspException, IOException {
		WebResourceData resource = index == null? webResourcesRepository.getResourceData(bundle) : webResourcesRepository.getResourceData(bundle, index, browserFileName);
		urlBuilder.replaceParameter(ResourcesAction.BUNDLE, bundle);
		urlBuilder.replaceParameter(ResourcesAction.INDEX, index == null? ResourcesAction.INDEX_ALL : index);
		urlBuilder.replaceParameter(ResourcesAction.RESOURCE, browserFileName);
		urlBuilder.replaceParameter(ResourcesAction.MD5, resource.getMd5());
		printTag();
	}

	
	@Override
	protected void endTag() {
		try {
			super.endTag();
			if (ieLessThan != null) {
				out.print("<![endif]-->");
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	protected String getContents() {
		return contents;
	}
	
	/**
	 * The Action to invoke. If specified, the link will be calculated using the 
	 * action/event pair.   
	 * @param action The unqualified action name, with the "Action" suffix removed. The fully qualified class name can also be used.
	 */
	@Attribute
	public void setAction(String action) {
		this.urlBuilder.setAction(action);
	}

	/**
	 * The Event to invoke. If specified, the link will be calculated using the 
	 * action/event pair. Any not-null nested <code>param</code> tag or <code>param-</code>
	 * prefixed attribute will be appended.  
	 */
	@Attribute
	public void setEvent(String event) {
		this.urlBuilder.setEvent(event);
	}
	
	public void setResource(String resourceName) {
		this.resource = resourceName;
	}

	public AbstractWebResourceTag addParameter(String name, Object value) {
		urlBuilder.addParameter(name, value);
		return this;
	}
	
	public void setIeLessThan(String ie) {
		if (ie == null) {
			this.ieLessThan = null;
		} else {
			this.ieLessThan = "any".equals(ie)? IE_ANY : Integer.parseInt(ie);
		}
	}

	protected void setRawUrl(String rawUrl) {
		urlBuilder.setRawUrl(rawUrl);
	}

	public UrlBuilder getUrl() {
		return urlBuilder;
	}

	public boolean isDevelopment() {
		return development;
	}

	public void setDevelopment(boolean development) {
		this.development = development;
	}

	public String getResource() {
		return resource;
	}

	public WebResourcesRepository getWebResourcesRepository() {
		return webResourcesRepository;
	}

	public void setWebResourcesRepository(WebResourcesRepository resourcesRepository) {
		this.webResourcesRepository = resourcesRepository;
	}

	protected String getType() {
		return type;
	}

	protected void setType(String type) {
		this.type = type;
	}

	protected UrlBuilder getUrlBuilder() {
		return urlBuilder;
	}

}
