/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.servlet.jsp.JspException;

import org.loom.log.Log;
import org.loom.servlet.Scope;
import org.loom.servlet.names.SessionAttributeNames;
import org.loom.tags.AbstractTag;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Cache the tag contents. The tag body will be skipped if there is usable content in the cache. 
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=false,
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;l:cache id=\"cached-menu\">\n" +
			"	&lt;l:menu> ... &lt;/l:menu>\n" +
			"&lt;/l:cache>"
	)
public class CacheTag extends AbstractTag {

	/** the key to be used to identify this cache entry */ 
	@Attribute(required=true)
	private String id;
	
	/** true to use separate cache contents per language. Default true */
	@Attribute
	private boolean useLocale = true;
	
	private Scope scope = Scope.SESSION;
	
	private static Log log = Log.getLog();

	@Override
	@SuppressWarnings("unchecked")
	public void doTagImpl() throws JspException, IOException {
		Map<String, String> container = (Map<String, String>) getScopedAttribute(SessionAttributeNames.CACHED_CONTAINER, scope);
		if (container == null) {
			// do not worry about race conditions, since it is just cached data 
			container = new ConcurrentHashMap<String, String>();
			setScopedAttribute(SessionAttributeNames.CACHED_CONTAINER, container, scope);
		}
		String key = useLocale? getMessagesRepository().getLocale().toString() + "-" + id : id;
		String contents = container.get(key);
		if (contents == null) {
			log.debug("Cache hit failed for key ", key);
			contents = doBufferedBody();
			container.put(key, contents);
		} else {
			log.debug("Using cached value for key ", key);
		}
		out.print(contents);
	}
	
	public void setId(String id) {
		this.id = id;
	}

	public void setUseLocale(boolean useLocale) {
		this.useLocale = useLocale;
	}

	/** 
	 * Scope where the cached value will be stored.
	 * Possible values are <code>session</code> (default) and <code>application</code>.  
	 */
	@Attribute
	public void setScope(String scope) {
		this.scope = Scope.fromString(scope);
		if (this.scope != Scope.SESSION && this.scope != Scope.APPLICATION) {
			throw new IllegalArgumentException("Illegal scope value: " + scope + " only session and application are allowed");
		}
	}

}
