/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;

import javax.servlet.jsp.JspException;

import org.loom.tags.AbstractHtmlTag;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;
import org.loom.url.ParameterContainer;
import org.loom.url.UrlBuilder;
import org.loom.util.HtmlSanitizer;


/**
 * <a href="http://www.w3.org/TR/html4/struct/objects.html#edef-IMG">HTML img tag</a>.
 *
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.SCRIPTLESS,
		example=
			"&lt;img action=\"File\" param-file.id=\"${file.id}\"/>\n" +
			"&lt;img src=\"/foo/bar.gif\"/>"
	)
public class ImgTag extends AbstractHtmlTag implements ParameterContainer {

	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-align-IMG">HTML attribute</a> */
	@Attribute
	private String align;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-alt">HTML attribute</a> */
	@Attribute
	private String alt;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-longdesc-IMG">HTML attribute</a> */
	@Attribute
	private String longdesc;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-usemap">HTML attribute</a> */
	@Attribute
	private String usemap;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-vspace">HTML attribute</a> */
	@Attribute
	private String vspace;
	
	/** <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-src-IMG">HTML attribute</a> */
	/** action and event are an alternative to using src */
	private UrlBuilder url = new UrlBuilder();
	
	/** true to translate the alt attribute, default true */
	@Attribute
	private boolean translateAlt = true;

	@Override
	public void doTagImpl() throws JspException, IOException {
		// to get any possible nested param for the url; other than that, the body gets discarded
		doBufferedBody(); 
		if (getTitle() == null) {
			setTitle(url.calculateTitle());
		}
		out.print("<img");
		out.printAttribute("src", url.getURL());
		out.printAttribute("alt", HtmlSanitizer.sanitize(getTranslatedAlt()));
		out.printAttribute("align", align);
		out.printAttribute("longdesc", repository.getString(longdesc));
		out.printAttribute("usemap", usemap);
		out.printAttribute("vspace", vspace);
		printParentAttributes();
		out.print("/>");
	}
	
	private String getTranslatedAlt() {
		return translateAlt? repository.getString(alt) : alt;
	}

	public void setAlt(String alt) {
		this.alt = alt;
	}

	public void setAlign(String align) {
		this.align = align;
	}

	public void setLongdesc(String longdesc) {
		this.longdesc = longdesc;
	}

	public void setUsemap(String usemap) {
		this.usemap = usemap;
	}

	public void setVspace(String vspace) {
		this.vspace = vspace;
	}

	/**
	 * The action this link points to. Either this or src must be specified
	 */
	@Attribute
	public void setAction(String action) {
		url.setAction(action);
	}

	/**
	 * The event this link points to. If action is specified but event is not, the
	 * default event will be assumed. If the action does not define a default event,
	 * an exception will be thrown.
	 * Either action/event or src must be set.
	 */
	@Attribute
	public void setEvent(String event) {
		url.setEvent(event);
	}
	
	/** 
	 * <a href="http://www.w3.org/TR/html4/struct/objects.html#adef-src-IMG">HTML attribute</a>
	 * The target url. Either this or the action / event pair must be set.
	 */
	@Attribute
	public void setSrc(String src) {
		url.setRawUrl(src);
	}
	
	public ImgTag addParameter(String name, Object value) {
		url.addParameter(name, value);
		return this;
	}

	public boolean isTranslateAlt() {
		return translateAlt;
	}

	public void setTranslateAlt(boolean translateAlt) {
		this.translateAlt = translateAlt;
	}
	
	public UrlBuilder getUrlBuilder() {
		return url;
	}

}
