/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.util.Set;

import javax.servlet.jsp.JspException;

import org.loom.config.ConfigProperties;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.tags.AbstractFormInputTag;
import org.loom.util.PropertyUtils;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;


/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#edef-INPUT">HTML checkboxes</a>. 
 * <p>
 * For the normal use, value is ignored and the checkbox will render a "checked" attribute 
 * if the associated property is not null.
 * When associating with a Set property, the name attribute should specify the value to get/set from the set 
 * as an index; if the Set contains the value, the checkbox will be checked, else it will not. On form submission,
 * the Set will be populated with the elements that have been checked in the browser. 
 * </p>
 * <p>
 * A hidden field will be generated after the checkbox to work around the fact that browsers do not submit unchecked checkboxes.
 * </p>
 * @author icoloma
 * 
 */
@Tag(
	dynamicAttributes=true, 
	bodyContent=BodyContent.EMPTY,
	example=
		"&lt;l:inputCheckbox name=\"mortgage.starred\"/>\n" +
		"&lt;c:forEach var=\"item\" items=\"${items}\" >\n" +
		"	&lt;li>&lt;l:inputCheckbox name=\"selectedItems[${item.id}]\" />\n" +
		"	   Select item ${item.name}\n" +
		"	&lt;/li>\n" +
		"&lt;/c:forEach>"

)
public class InputCheckboxTag extends AbstractFormInputTag {

	private static String DEFAULT_CSS = ConfigProperties.getInstance().getDefaultCss(InputCheckboxTag.class, "checkbox");

	/** true if this checkbox is bound to a Set property (lazily initialized) */
	private Boolean multipleValue;
	
	/** true if this checkbox is checked (lazily initialized) */
	private Boolean checkedValue;
	
	public InputCheckboxTag() {
		setCssClass(DEFAULT_CSS);
		this.setLabelPosition(LabelPosition.RIGHT);
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		// if property type is Set, generate an index value automatically
		boolean checked = isChecked();
		
		// according to the documentation of getParameter, the value that is returned is the first to be found. 
		// Therefore, establishing the first checkbox and then the hidden.
		//  print the tag
		out.print("<input type=\"checkbox\"");
		printParentAttributes();
		if (checked) {
			out.print(" checked");
		}
		if (!isMultiple()) { // do not generate value for single-valued checkboxes
			out.printAttribute("value", getValue());
		}
		out.print(" />");
		
		// LOOM-32: unchecked checkboxes are not sent by the browser
		out.printHiddenField(getName(), "false");
	}
	
	/**
	 * @return true if the checkbox is selected
	 */
	protected boolean isChecked() {
		if (checkedValue == null) {
			checkedValue = false;
			ParsedAction parsedAction = request.getParsedAction();
			if (parsedAction != null) {
				Object propertyValue = parsedAction.getPropertyAsObject(getName());
				if (propertyValue != null) {
					checkedValue = ((Boolean)propertyValue).booleanValue();
				}
			}
		}
		return checkedValue;
	}
	
	/**
	 * Return true if the checkbox is bound to a Set and false if not
	 */
	protected boolean isMultiple() {
		if (multipleValue == null) {
			Event event = getForm().getTargetEvent();
			Class<?> clazz = event.getPropertyClass(getNormalizedName());
			if (Boolean.class.equals(clazz) || Boolean.TYPE.equals(clazz)) {
				multipleValue = false;
			} else if (Set.class.isAssignableFrom(clazz)) {
				multipleValue = true;
			} else {
				throw new IllegalArgumentException("InputCheckboxTag does not know how to handle property of class " + clazz.getName());
			}
		}
		return multipleValue;
	}
	
	/**
	 * Returns the translation of the checkbox appended to the property path.
	 * if label is present it will be used as prefix inestead of name
	 * Example: if the name is mortgage.fixedIncome and the value is true, it will 
	 * return the translation of "mortgage.fixedIcome.true"
	 * if label is "foobar"; will return the translation of "foobar.true"
	 * 
	 */
	@Override
	public String renderAsText() {
		boolean checked = isChecked();
		if (isMultiple()) { // Set
			if (checked) {
				return isTranslateLabel()? repository.guessString(getIndex()) : getIndex(); 
			}
			return "";
		} else { // boolean
			String value = PropertyUtils.concatPropertyPath(getLabel(), String.valueOf(checked));
			return repository.guessString(value);
		}
	}
	
	private String getIndex() {
		int index = getName().lastIndexOf('[');
		return index == -1? null : getName().substring(index + 1, getName().lastIndexOf(']'));
	}

	/**
	 * Multiple checkboxes set to renderAsText should not render a label
	 */
	@Override
	public boolean shouldRenderLabel() {
		if (getRenderLabel() == null && Boolean.TRUE.equals(getRenderAsText()) && isMultiple()) {
			return false;
		}
		return super.shouldRenderLabel();
	}
	

}
