/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.core;

import java.io.IOException;
import java.math.BigDecimal;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.joda.time.DateTime;
import org.loom.config.ConfigProperties;
import org.loom.servlet.names.HtmlExtendedAttributeNames;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;
import org.loom.util.HtmlSanitizer;


/**
 * <a href="http://www.w3.org/TR/html4/interact/forms.html#edef-INPUT">HTML text input field</a>.
 * Each form field will generate automatically details such as the maxlength and size attributes, 
 * and the corresponding decorator will generate the input field label.
 * @author icoloma
 *
 */
@Tag(
		dynamicAttributes=true, 
		bodyContent=BodyContent.EMPTY,
		example=
			"&lt;l:inputText name=\"mortgage.name\"/>\n" +
			"&lt;l:inputText name=\"customer.friends[2].name\"/>\n" +
			"&lt;l:inputText name=\"customer.friendsMap[john].name\"/>"
	)
public class InputTextTag extends AbstractTextFieldTag implements NumberFieldTag {

	private static String DEFAULT_CSS = ConfigProperties.getInstance().getDefaultCss(InputTextTag.class, null);

	/** default max size when calculating automatically from the maxlength */
	public static int DEFAULT_MAX_SIZE = ConfigProperties.getInstance().getPropertyAsInteger("css.defaultMaxSize", 40);

	/** <a href="http://www.w3.org/TR/html4/interact/forms.html#adef-size-INPUT">HTML attribute</a> */
	@Attribute
	private Integer size;

	/** 
	 * The <a href="http://msdn2.microsoft.com/en-us/library/ms533486(VS.85).aspx">non-standard attribute</a> to enable/disable browser autocomplete for any field (Firefox and IE).
	 * If not set, it will use the value specified in the container form tag.
	 */
	@Attribute
	private Boolean autocomplete;
	
	/** 
	 * <a href="http://www.whatwg.org/specs/web-forms/current-work/#min">Minimum value</a>
	 * <strong>HTML 5</strong>. Note that this attribute is set by validators and cannot be 
	 * manually set in the JSP page. 
	 */	
	private String min;
	
	/** 
	 * <a href="http://www.whatwg.org/specs/web-forms/current-work/#max">Maximum value</a>
	 * <strong>HTML 5</strong>. Note that this attribute is set by validators and cannot be 
	 * manually set in the JSP page. 
	 */	
	private String max;
	
	public InputTextTag() {
		setCssClass(DEFAULT_CSS);
		setType("text");
	}
	
	@Override
	public void doTagImpl() throws JspException, IOException {
		out.print("<input");
		printParentAttributes();
		out.printAttribute(HtmlExtendedAttributeNames.MIN, min);
		out.printAttribute(HtmlExtendedAttributeNames.MAX, max);
		out.printAttribute("value", HtmlSanitizer.sanitize(getValue()));
		if (size == null && maxlength != null) {
			size = Math.min(maxlength, DEFAULT_MAX_SIZE);
		}
		out.printAttribute("size", size);
		if (autocomplete !=  null) {
			out.printAttribute("autocomplete", autocomplete? "on" : "off");
		}
		out.print("/>");
	}
	
	/**
	 * Set the excludeMin attribute of the input tag. If excludeMin == false 
	 * (the default) the value will be ignored.
	 */
	public void setExcludeMin(boolean excludeMin) {
		if (excludeMin) {
			setExtendedAttribute(HtmlExtendedAttributeNames.EXCLUDE_MIN, excludeMin);
		}
	}
	
	/**
	 * If size is explicitely set, abbreviate the output string to the expected length
	 */
	@Override
	public String renderAsText() {
		String v = super.renderAsText();
		return size == null? v : StringUtils.abbreviate(v, size);
	}
	
	/**
	 * Set the excludeMax attribute of the input tag. If excludeMax == true 
	 * (the default) the value will be ignored.
	 */
	public void setExcludeMax(boolean excludeMax) {
		if (!excludeMax) {
			setExtendedAttribute(HtmlExtendedAttributeNames.EXCLUDE_MAX, excludeMax);
		}
	}
	
	public void setPrecision(Integer precision) {
		setExtendedAttribute(HtmlExtendedAttributeNames.PRECISION, precision);
	}
	
	public void setScale(Integer scale) {
		setExtendedAttribute(HtmlExtendedAttributeNames.SCALE, scale);
	}
	
	public void setSize(Integer size) {
		this.size = size;
	}

	public Boolean getAutocomplete() {
		return autocomplete;
	}

	public void setAutocomplete(Boolean autocomplete) {
		this.autocomplete = autocomplete;
	}

	public void setMin(BigDecimal min) {
		if (min != null) {
			this.min = min.toString();
		}
	}

	public void setMax(BigDecimal max) {
		if (max != null) {
			this.max = max.toString();
		}
	}

	/**
	 * Sets the min of a date field. Dates are converted toISO8601 per the spec.
	 * TODO: should timezones be considered here?
	 */
	public void setMin(DateTime date) {
		if (date != null) {
			this.min = date.toString();
		}
	}
	
	public void setMax(DateTime date) {
		if (date != null) {
			this.max = date.toString();
		}
	}

	public String getMin() {
		return min;
	}

	public String getMax() {
		return max;
	}
	
	
}
