/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.decorator;

import java.util.HashMap;
import java.util.Map;

import org.loom.tags.AbstractDecoratedTag;
import org.loom.tags.AbstractFormInputTag;
import org.loom.tags.LoomTag;
import org.loom.tags.core.InputRadioTag;
import org.loom.tags.menu.InlineMenuDecorator;
import org.loom.tags.menu.NestedMenuDecorator;
import org.loom.tags.tabs.DefaultTabDecorator;
import org.loom.tags.tabs.TabTag;

/**
 * Repository of {@link TagDecorator} instances.
 * For each tag class, there is a configured TagDecorator instance. If none,
 * each parent class in the hierarchy is searched for a configured Decorator.
 * If none is found, returns null.
 * @author icoloma
 */
public class TagDecoratorRepository {

	private Map<String, TagDecorator<? extends LoomTag>> mappings;
	
	public TagDecoratorRepository() {
		mappings = new HashMap<String, TagDecorator<? extends LoomTag>>();
		
		addMapping(AbstractFormInputTag.class, new AbstractFormInputDecorator<AbstractFormInputTag>());
		addMapping(InputRadioTag.class, new InputRadioDecorator());
		addMapping(TabTag.class, new DefaultTabDecorator());
		//addMapping("echo", new PrintHiddenFieldDecorator()); // @deprecated to be removed in 1.1
		addMapping("print", new PrintHiddenFieldDecorator());
		addMapping("inline", new InlineMenuDecorator());
		addMapping("link-only", new LinkOnlyDecorator());
		addMapping("nested", new NestedMenuDecorator());
		addMapping("read-only", new ReadOnlyDecorator<AbstractFormInputTag>());
	}
	
	/**
	 * Adds a mapping for a tag class
	 */
	public void addMapping(Class<? extends LoomTag> clazz, TagDecorator<? extends LoomTag> decorator) {
		mappings.put(clazz.getName(), decorator);
	}

	/**
	 * Adds a mapping with a specific name
	 */
	public void addMapping(String name, TagDecorator<? extends LoomTag> decorator) {
		mappings.put(name, decorator);
	}

	public TagDecorator<? extends LoomTag> getDecorator(Class<?> tagClass) {
		while (!AbstractDecoratedTag.class.equals(tagClass) && tagClass != null) {
			TagDecorator<? extends LoomTag> result = getDecorator(tagClass.getName());
			if (result != null) {
				return result;
			}
			tagClass = tagClass.getSuperclass();
		}
		return null;
	}
	
	public TagDecorator<? extends LoomTag> getDecorator(String key) {
		return mappings.get(key);
	}
	
	public void setMappings(Map<String, TagDecorator<? extends LoomTag>> mappings) {
		this.mappings = mappings;
	}
	
	public Map<String, TagDecorator<? extends LoomTag>> getMappings() {
		return mappings;
	}

	public void setAdditionalMappings(Map<String, TagDecorator<? extends LoomTag>> additionalMappings) {
		this.mappings.putAll(additionalMappings);
	}
}
