/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import java.io.IOException;
import java.util.Locale;

import javax.servlet.jsp.JspException;

import org.apache.commons.lang.StringUtils;
import org.loom.util.LocaleUtils;
import org.tldgen.annotations.Attribute;
import org.tldgen.annotations.BodyContent;
import org.tldgen.annotations.Tag;

/**
 * Menu to select the current locale.
 * 
 * @author icoloma
 */
@Tag(
		dynamicAttributes=true,
		bodyContent=BodyContent.SCRIPTLESS,
		example= "&lt;l:localeMenu id=\"languages\" locales=\"en, es\"/>"
	)
public class LocaleMenuTag extends MenuTag {
	
	/** the list of available locales, separated by commas */
	@Attribute(required=true)
	private String locales;
	
	/** true if each locale is translated by itself (instead of the current request locale)
	 * before rendering, e.g. display Español instead of Spanish for a user that is browsing 
	 * in english. Default true */
	@Attribute
	private boolean selfTranslated = true;
	
	/** the Action and event that will be invoked to change the locale, defaults to LocaleAction and its default event */ 
	@Attribute
	private String action = "Locale"; 

	/** the Action and event that will be invoked to change the locale, defaults to LocaleAction and its default event */ 
	@Attribute
	private String event;
	
	public LocaleMenuTag() {
		setId("locale-menu");
		decorator = "inline";
		setItemSeparator(" | ");
	}
	
	/**
	 * Neither this or MenuItemTag perform any rendering, the whole work is done by the Decorator.
	 * These tags only assemble a structure of {@link MenuItemData}.
	 */
	@Override
	public void doTagImpl() throws IOException, JspException {
		LocaleMenuData menu = new LocaleMenuData();
		menu.setId(getId());
		menu.setCssClass(getCssClass());
		(menu).setSelfTranslated(selfTranslated);
		String[] l = StringUtils.split(locales, ",");
		for (String localeName : l) {
			Locale locale = LocaleUtils.toLocale(localeName.trim());
			LocaleMenuItemData item = new LocaleMenuItemData();
			item.setLocale(locale);
			item.setSelfTranslated(selfTranslated);
			item.getUrlBuilder().setAction(action);
			item.getUrlBuilder().setEvent(event);
			menu.addChild(item);
		}
		setMenu(menu);
		doBody();
	}
	
	public void setLocales(String locales) {
		this.locales = locales;
	}

	public boolean isSelfTranslated() {
		return selfTranslated;
	}

	public void setSelfTranslated(boolean selfTranslated) {
		this.selfTranslated = selfTranslated;
	}

	public void setAction(String actionName) {
		this.action = actionName;
	}

	public void setEvent(String event) {
		this.event = event;
	}

	public String getLocales() {
		return locales;
	}

}
