/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.tags.menu;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.loom.action.Action;
import org.loom.config.Config;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.Event;
import org.loom.mapping.ParsedAction;
import org.loom.servlet.LoomServletRequest;
import org.loom.url.UrlBuilder;

/**
 * Holds the first row of MenuItemData instances of a normal menu.
 * @author icoloma
 */
public class MenuData extends AbstractMenuData {

	private static final long serialVersionUID = 1L;

	/** 
	 * to easily locate the currently selected menuItem. An action/event
	 * pair is used as key, and they can match more than one 
	 * MenuItem, in such case we will use the parameters to differenciate 
	 * between them 
	 */
	private Map<MenuItemHashKey, MenuItemData[]> lookupByActionAndEvent = new HashMap<MenuItemHashKey, MenuItemData[]>();
	
	/**
	 * There are menu items that are linked to jsp pages (just url, no action or event).
	 * This map keeps those indexed by url
	 */
	private Map<String, MenuItemData> lookupByUrl = new HashMap<String, MenuItemData>();
	
	/**
	 * There are menu items that do not point anywhere, or simply the user may want to address a menuItem by id
	 */
	private Map<String, MenuItemData> lookupById = new HashMap<String, MenuItemData>();
	
	@Override
	public void addChild(MenuItemData item) {
		super.addChild(item);
		addFastLookup(item);
	}
	
	/**
	 * Add an item and all its children to the FastLookupMap
	 */
	private void addFastLookup(MenuItemData item) {
		if (item.getUrlBuilder() != null) {
			// adds the current item
			UrlBuilder urlBuilder = item.getUrlBuilder();
			ActionMapping action = urlBuilder.getActionMapping();
			String url = urlBuilder.getRawUrl();
			if (action != null) { // action/event pair
				createFastLookupNode(action.getShortName(), urlBuilder.getEvent().getName(), item);
			} else if (url != null && !url.startsWith("http://")) { // local url 
				lookupByUrl.put(url, item);
			}

			// process the aliases
			String[] aliases = item.getAliases();
			if (aliases != null) {
				// add each alias as an event that would mark this option as selected
				for (String alias : aliases) {
					if (alias.indexOf('/') == -1 && action != null) { // the alias is an event
						createFastLookupNode(action.getShortName(), alias, item);
					} else { // the alias is a JSP page
						lookupByUrl.put(alias, item);
					}
				}
			}

		}
		
		if (item.getId() != null) {
			lookupById.put(item.getId(), item);
		}
		
		// adds its children
		if (item.hasChildren()) {
			for (MenuItemData child : item.getChildren()) {
				addFastLookup(child);
			}
		}
	}

	/**
	 * Create a fast lookup node for the specified action and event
	 * @param item
	 */
	private void createFastLookupNode(String action, String event, MenuItemData item) {
		MenuItemHashKey key = new MenuItemHashKey(action, event);
		MenuItemData[] items = lookupByActionAndEvent.get(key);
		if (items == null) {
			lookupByActionAndEvent.put(key, new MenuItemData[] { item });
		} else {
			items = (MenuItemData[]) ArrayUtils.add(items, item);
			lookupByActionAndEvent.put(key, items);
		}
	}
	
	@Override
	public MenuItemData getSelectedItem(LoomServletRequest request) {
		
		// if any action or interceptor has registered a selected menu item
		SelectedMenuItemLocator selected = SelectedMenuItemLocator.getSelectedMenuItem(request, getId());
		MenuItemData result = null;
		if (selected != null) {
			result = getSelectedItem(selected);
		} else {
			// search the current action + id
			ParsedAction action = request.getParsedAction();
			if (action != null) {
				result = getSelectedItem(action);
			}
		}
		
		// if everything else fails, maybe it's a jsp
		if (result == null) {
			String uri = request.getServletPath(); 
			result = lookupByUrl.get(uri);
		}
		
		return result;
	}

	private MenuItemData getSelectedItem(ParsedAction action) {
		MenuItemHashKey key = new MenuItemHashKey(action);
		MenuItemData[] candidates = lookupByActionAndEvent.get(key);
		if (candidates != null) {
			for (MenuItemData item : candidates) {
				
				// check if any of the aliases is a match
				String[] aliases = item.getAliases();
				UrlBuilder urlBuilder = item.getUrlBuilder();
				if (aliases != null) {
					ActionMapping candidateAction = urlBuilder.getActionMapping();
					for (String alias : aliases) {
						if (action.matches(candidateAction, alias)) {
							return item;
						}
					}
				}
				
				// check if the url matches
				if (action.matches(urlBuilder)) {
					return item;
				}
			}
		}
		return null;
	}

	/**
	 * @return the selected menu item 
	 */
	private MenuItemData getSelectedItem(SelectedMenuItemLocator selected) {
		// search by id
		String selectedId = selected.getMenuItemId();
		if (selectedId != null) {
			MenuItemData item = lookupById.get(selectedId);
			if (item == null) {
				throw new IllegalArgumentException("MenuItem " + selectedId + " could not be located in menu " + getId());
			}
			return item;
		} 
		
		// search the selected action + event
		MenuItemHashKey key = new MenuItemHashKey(selected.getActionClass(), selected.getEvent());
		MenuItemData[] candidates = lookupByActionAndEvent.get(key);
		if (ArrayUtils.isEmpty(candidates)) {
			throw new IllegalArgumentException("MenuItem " + selected + " could not be located in menu " + getId());
		}
		return candidates[0];
	}
	
	/**
	 * Holds the data of a menuItem for fast lookups to get the currently selected MenuItemData instance
	 * @author icoloma
	 */
	public static class MenuItemHashKey implements Serializable {

		private Class<? extends Action> action;
		private String event;

		private int hashCode; 
		
		private static final long serialVersionUID = 1L;
		
		public MenuItemHashKey(String action, String event ) {
			ActionMapping mapping = Config.getInstance().getActionMappingRepository().getActionMappingByName(action);
			setAction(mapping.getActionClass());
			if (event == null) {
				Event defEvent = mapping.getDefaultEvent();
				if (defEvent == null) {
					throw new IllegalArgumentException("Could not instantiate MenuItem for action " + action + " because no event was specified and the action does not have a default event");
				}
				event = defEvent.getName();
			}
			setEvent(event);
		}
		
		public MenuItemHashKey(Class<? extends Action> actionClass, String event) {
			setAction(actionClass);
			setEvent(event);
		}
		
		public MenuItemHashKey(ParsedAction action) {
			this(action.getActionMapping().getActionClass(), action.getEvent().getName());
		}
		
		@Override
		public int hashCode() {
			return hashCode;
		}

		@Override
		public boolean equals(Object obj) {
			if (obj == null || !(obj instanceof MenuItemHashKey)) {
				return false;
			}
			final MenuItemHashKey other = (MenuItemHashKey) obj;
			
			return compare(action, other.action) && StringUtils.equals(event, other.event);
		}
		
		private boolean compare(Class<? extends Action> c1, Class<? extends Action> c2) {
			if (c1 == null) {
				return c2 == null;
			} else {
				return c1.equals(c2);
			}
		}

		private void setAction(Class<? extends Action> action) {
			this.action = action;
			hashCode += this.action.hashCode();
		}

		private void setEvent(String event) {
			this.event = event;	
			if (event !=  null) {
				hashCode += this.event.hashCode();
			}
		}
		
	}

}
