/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.url;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.loom.action.Action;
import org.loom.config.Config;
import org.loom.converter.Converter;
import org.loom.mapping.ActionMapping;
import org.loom.mapping.ActionMappingRepository;
import org.loom.mapping.Event;
import org.loom.servlet.FlashContainer;
import org.loom.servlet.LoomServletRequest;
import org.loom.servlet.LoomServletRequestImpl;
import org.loom.servlet.names.RequestParameterNames;
import org.loom.util.PropertyUtils;
import org.loom.util.StringUtils;
import org.loom.validator.PropertyValidator;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.LinkedListMultimap;
import com.google.common.collect.ListMultimap;

/**
 * Creates a URL from a action/event pair, or (alternatively) a URL with
 * the web context prepended.
 * 
 * @author icoloma
 */
public class UrlBuilder implements ParameterContainer, Cloneable {

	/** insecure (http) scheme */
	public static String INSECURE_SCHEME = "http";
	
	/** secure (https) scheme */
	public static String SECURE_SCHEME = "https";
	
	/** default insecure (http) port */
	public static Integer INSECURE_PORT = Integer.valueOf(80);
	
	/** default secure (https) port */
	public static Integer SECURE_PORT = Integer.valueOf(443);

	/** regex to parse string URLs */
	private static final Pattern urlPattern = Pattern.compile("^(https?:\\/\\/[^/]+)?(\\/[^\\?&#]*)(\\?[^#]+)?(#.+)?$");
	
	/** action mapping */
	private ActionMapping actionMapping;
	
	/** the event to be invoked, as an object */
	private Event event;
	
	/** the name of the event to be called */
	private String eventName;
	
	/** the url, if specified by hand */
	private String rawUrl;
	
	/** the url arguments */
	private ListMultimap<String, String> params = LinkedListMultimap.create();
	
	/** the url hash, if any (the part after the '#' character of the URI) */
	private String hash;
	
	/** not null to override the request hostname */
	private String hostname;
	
	/** not null to override the request port */
	private Integer port;

	/** 
	 * Set this to not null to override the request scheme (e.g.: "http", "https") 
	 * Set to the empty String to generate a protocol-less URL, e.g. "//myhost.com/resource.html"
	 */
	private String scheme;
	
	/** 
	 * true to append to the generated URL all current request parameters, default false.
	 * Parameters added to this {@link UrlBuilder} instance will override the value included in the request.
	 */
	private boolean includeCurrentRequestParameters;

	/**
	 * true to encode the query string parameters. defaults to true [LOOM-188] 
	 */
	private boolean encodeParameters = true;
	
	/** true to include web context path when converting to string */
	private boolean includeContextPath = true;
	
	private LoomServletRequest request;
	
	public UrlBuilder() {
		this(LoomServletRequestImpl.getThreadLocal());
	}
	
	public UrlBuilder(LoomServletRequest request) {
		this.request = request;
	}
	
	/**
	 * @deprecated use LoomServletRequest.createUrl(actionClass, event) instead
	 */
	public UrlBuilder(Class<? extends Action> actionClass, String event) {
		this(LoomServletRequestImpl.getThreadLocal(), actionClass, event);
	}
	
	public UrlBuilder(LoomServletRequest request, Class<? extends Action> actionClass, String event) {
		this.request = request;
		setAction(actionClass);
		setEvent(event);
	}
	
	/**
	 * Parse a url and extracts any parameters or hash 
	 * @param url the url to parse
	 * @deprecated use LoomServletRequest.createUrl(url) instead
	 */
	public UrlBuilder(String url) {
		this(LoomServletRequestImpl.getThreadLocal(), url);
	}
	
	public UrlBuilder(LoomServletRequest request, String url) {
		this.request = request;
		Matcher m = urlPattern.matcher(url);
		if (!m.matches()) {
			throw new IllegalArgumentException("Cannot parse url: " + url);
		}
		this.includeContextPath = false;
		
		// extract scheme, hostname and port
		String s = m.group(1);
		if (s != null) { 
			scheme = s.charAt(4) == 's'? "https" : "http";
			int hostnameStartPos = scheme.length() + 3;
			int pos = s.indexOf(':', hostnameStartPos);
			if (pos == -1) { // no port specified
				hostname = s.substring(hostnameStartPos);
			} else {
				hostname = s.substring(hostnameStartPos, pos);
				port = Integer.parseInt(s.substring(pos + 1));
			}
		}
		
		// extract URL path
		rawUrl = m.group(2);
		
		// process the parameters
		s = m.group(3);
			if (s != null) {
			String[] a = StringUtils.split(s.substring(1), '&');
			for (String param: a) {
				String[] nameAndValue = StringUtils.split(param, '=');
				addParameter(nameAndValue[0], nameAndValue.length == 1? "" : nameAndValue[1]);
			}
		}
			
		//  process the hash
		s = m.group(4);
		if (s != null) {
			hash = s.substring(1);
		}
	}
/*	
	@SuppressWarnings("unchecked")
	public void readExternal(ObjectInput in) throws IOException,
			ClassNotFoundException {
		String actionName = (String) in.readObject();
		if (actionName != null) {
			setAction(actionName);
		}
		setEvent((String) in.readObject());
		setRawUrl((String) in.readObject());
		params = (ListMultimap<String, String>) in.readObject();
		setHash((String) in.readObject());
	}

	public void writeExternal(ObjectOutput out) throws IOException {
		out.writeObject(actionMapping == null? null : actionMapping.getActionClass().getName());
		out.writeObject(eventName);
		out.writeObject(rawUrl);
		out.writeObject(params);
		out.writeObject(hash);
	}
	*/
	@Override
	public UrlBuilder clone() {
		try {
			UrlBuilder copy = (UrlBuilder) super.clone();
			copy.params = ArrayListMultimap.create(params);
			return copy;
		} catch (CloneNotSupportedException e) {
			// should not happen
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * @param normalizedName the (not-normalized) property name for which the validations are going to be retrieved
	 * @return the list of validations applicable to the supplied propertyName, null if none 
	 */
	public Set<PropertyValidator> getValidations(String normalizedName) {
		return event == null? null : event.getValidatorsFor(normalizedName);
	}

	/**
	 * Calculates a valid title, if none is provided.
	 * Concatenates the action and event, if any, to provide a valid candidate.
	 */
	public String calculateTitle() {
		return event == null? null : event.getPrintName();
	}
	
	/**
	 * @deprecated use getURL() instead
	 */
	public String getURL(LoomServletRequest request) {
		return getURL();
	}
	
	/**
	 * Calculate the output URL 
	 */
	public String getURL() {

		// append scheme, hostname and port
		StringBuilder result = new StringBuilder(100);
		if (StringUtils.isEmpty(rawUrl) || rawUrl.charAt(0) == '/') { // if the URL is not relative
			if (scheme != null || hostname != null || port != null) {
				if (!hasEvent() && rawUrl == null) {
					throw new IllegalArgumentException("When specifying scheme/hostname/port, either action+event or rawUrl should be provided as well");
				}
				if (scheme == null) {
					scheme = request.isSecure()? SECURE_SCHEME : INSECURE_SCHEME;
				}
				if (hostname == null) {
					hostname = request.getServerName();
				}
				if (port == null && request.getServerName().equals(hostname) && 
						(SECURE_SCHEME.equals(scheme) && request.isSecure() && request.getServerPort() != SECURE_PORT || 
								INSECURE_SCHEME.equals(scheme) && !request.isSecure() && request.getServerPort() != INSECURE_PORT)) {
					port = request.getServerPort();
				}
				result.append(scheme).append(scheme.length() == 0? "//" : "://");
				result.append(hostname);
				if (port != null && 
						(!INSECURE_PORT.equals(port) || !INSECURE_SCHEME.equals(scheme)) && 
						(!SECURE_PORT.equals(port) || !SECURE_SCHEME.equals(scheme))) {
					result.append(':').append(port);
				}
			}
		}
		
		// add flash ID
		FlashContainer flash = FlashContainer.getNext(request, false);
		if (flash != null) {
			replaceParameter(RequestParameterNames.FLASH_ID, flash.getId());
		}
		
		// if required, add the current request params
		if (includeCurrentRequestParameters) {
			for (Map.Entry<String, String> param : request.getRequestParameters().getStringParametersEntrySet()) {
				if (!params.containsKey(param.getKey())){
					params.put(param.getKey(), param.getValue());
				}
			}
		}
		
		// append URI
		String contextPath = request.getContextPath();
		if (contextPath == null) { // workaround for a bug in JavaRebel
			contextPath = "";
		}
		if (hasEvent()) {
			result.append(contextPath);
			result.append(getEvent().toUrl(params)); 
		} else if (rawUrl != null) {
			if (includeContextPath &&
					rawUrl.length() > 0 && 
					rawUrl.charAt(0) == '/' && 
					(hostname == null || request.getServerName().equals(hostname))) { // absolute (not relative) URL in this host 
				result.append(contextPath);
			}
			result.append(rawUrl);
		} else if (hash == null) {
			throw new IllegalStateException("Neither action+event, rawUrl or hash have been set");
		}
		
		// append GET parameters
		addParams(request, result, params, event == null? null : event.getUriParameterNames());
		if (hash != null) {
			result.append('#').append(hash);
		}
		
		return result.toString();	
	}
	
	/**
	 * Add a list of params to the provided uri
	 * @param uri the uri to add parameters to
	 * @param params the list of UrlParameter
	 * @param exceptParams the list of parameters that should not be added
	 */
	private void addParams(LoomServletRequest request, StringBuilder uri, ListMultimap<String, String> params, Set<String> exceptParams) {
		try {
			boolean first = uri.indexOf("?") == -1;
			for (Map.Entry<String, String> entry: params.entries()) {
				String name = entry.getKey();
				if (exceptParams == null || !exceptParams.contains(name)) {
					if (first) {
						first = false;
						uri.append('?');
					} else {
						uri.append('&');
					}
					String value = entry.getValue();
					uri.append(name).append('=').append(encodeParameters ? URLEncoder.encode(value, "UTF-8") : value);
				}
			}
		} catch (UnsupportedEncodingException e) {
			throw new RuntimeException(e);
		}
	}
	
	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		builder.append("URLBuilder { ");
		if (rawUrl != null) {
			builder.append("url = ").append(rawUrl).append(", ");
		}
		if (actionMapping != null) {
			builder.append("action = ").append(actionMapping.getShortName()).append(", ");
		}
		if (event != null) {
			builder.append("event = ").append(event);
		}
		builder.append("} ");
		builder.append(params);
		return builder.toString();
	}

	/**
	 * Add a GET parameter to the generated URL. 
	 * @param name the name of the parameter
	 * @param value the value to add. If null, the parameter will not be added. If empty, it will.
	 */
	public UrlBuilder addParameter(String name, Object value) {
		if (value != null) { // empty values are ok, but null is not
			Converter converter = event == null? null : event.getConverter(PropertyUtils.normalize(name));
			String s = converter == null? value.toString() : converter.getAsText(value, request.getMessagesRepository());
			params.put(name, s);
		}
		return this;
	}
	
	/**
	 * Replace a parameter with a different value. If the parameter is multiple, all of them will get removed and a single one will be added. 
	 * @param name the name of the parameter to be replaced.
	 * @param value the new value of the parameter. If null, the parameter will just be removed.
	 * @return the list of removed parameters
	 */
	public List<String> replaceParameter(String name, Object value) {
		List<String> l = removeParameter(name);
		addParameter(name, value);
		return l;
	}
	
	/**
	 * Remove a GET parameter
	 * @param name the name of the Parameter to remove
	 * @return the list of removed parameters
	 */
	public List<String> removeParameter(String name) {
		return params.removeAll(name);
	}

	public UrlBuilder setAction(String actionClass) {
		ActionMappingRepository repository = Config.getInstance().getActionMappingRepository();
		setActionMapping(repository.getActionMappingByName(actionClass));
		return this;
	}
	
	public UrlBuilder setActionMapping(ActionMapping mapping) {
		this.actionMapping = mapping;
		this.event = eventName == null? actionMapping.getDefaultEvent() : actionMapping.getEvent(eventName);
		return this;
	}
	
	public UrlBuilder setAction(Class<? extends Action> actionClass) {
		ActionMappingRepository repository = Config.getInstance().getActionMappingRepository();
		setActionMapping(repository.getActionMappingByClass(actionClass));
		return this;
	}

	public UrlBuilder setEvent(String event) {
		this.eventName = event != null && event.length() == 0? null : event;
		if (actionMapping != null) {
			this.event = actionMapping.getEvent(eventName);
		}
		return this;
	}
	
	public UrlBuilder setEvent(Event event) {
		this.eventName = event.getName();
		this.event = event;
		this.actionMapping = event.getActionMapping();
		return this;
	}
	
	/**
	 * @return the value of the requested param, or null if not found.
	 */
	public String getParameter(String paramName) {
		List<String> l = params.get(paramName);
		return l.isEmpty()? null : l.get(0);
	}
	
	public ListMultimap<String, String> getParameters() {
		return params;
	}
	
	/**
	 * Clear the param list, but leave the action/event/rawUrl untouched
	 */
	public UrlBuilder clearParams() {
		params.clear();
		return this;
	}

	/**
	 * @return true if this url does not point to anywhere
	 */
	public boolean isEmpty() {
		return actionMapping == null && rawUrl == null;
	}

	public UrlBuilder setRawUrl(String url) {
		this.rawUrl = url;
		return this;
	}

	public String getRawUrl() {
		return rawUrl;
	}
	
	public Event getEvent() {
		if (event == null) {
			if (actionMapping == null) {
				throw new IllegalStateException("Neither event or action have been set");
			} else {
				throw new IllegalStateException("Event has not been set and " + actionMapping.getShortName() + " does not provide a default event to be used");
			}
		}
		return event;
	}
	
	/**
	 * @return true if this UrlBuilder has been set using a action/event pair, false if it has been set using rawUrl
	 */
	public boolean hasEvent() {
		return actionMapping != null || eventName != null;
	}

	public ActionMapping getActionMapping() {
		return actionMapping;
	}

	public String getHash() {
		return hash;
	}

	public UrlBuilder setHash(String hash) {
		this.hash = hash;
		return this;
	}

	public String getScheme() {
		return scheme;
	}

	public UrlBuilder setScheme(String scheme) {
		this.scheme = scheme;
		return this;
	}

	public String getHostname() {
		return hostname;
	}

	public UrlBuilder setHostname(String hostname) {
		this.hostname = hostname;
		return this;
	}

	public Integer getPort() {
		return port;
	}

	public UrlBuilder setPort(Integer port) {
		this.port = port;
		return this;
	}

	public boolean isEncodeParameters() {
		return encodeParameters;
	}

	public void setEncodeParameters(boolean encodeParameters) {
		this.encodeParameters = encodeParameters;
	}
	
	public boolean isIncludeCurrentRequestParameters() {
		return includeCurrentRequestParameters;
	}

	public UrlBuilder setIncludeRequestParameters(boolean addCurrentRequestParameters) {
		this.includeCurrentRequestParameters = addCurrentRequestParameters;
		return this;
	}

	public boolean isIncludeContextPath() {
		return includeContextPath;
	}

	public void setIncludeContextPath(boolean includeContextPath) {
		this.includeContextPath = includeContextPath;
	}
	
}
