/*
 * Copyright 2002-2006 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.loom.validator.date;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.joda.time.DateTime;
import org.loom.tags.FormInputTag;
import org.loom.tags.core.InputTextTag;
import org.loom.validator.AbstractPropertyValidator;
import org.loom.validator.ValidationRequest;


/**
 * Class that contains data and behaviour to validate one field.
 * This class and its descendants are thread-safe.
 * @author Ignacio Coloma
 */
public class DateValidator extends AbstractPropertyValidator {

	/** minimum allowed value */
    private DateValue minValue;
    
    /** maximum allowed value */
    private DateValue maxValue;

    /** true to exclude the minimum, default false */
    private boolean excludeMin = false;
    
    /** true to exclude the maximum, default false */
    private boolean excludeMax = true;
    
	/**
     * Validates one input value and adds to errors any validation error found 
     * @param v the value to validate
     */
    public void validate(ValidationRequest request, Object v) {
    	if (v != null) {
    		DateTime value = toDateTime(v);
        	if (minValue != null) {
    			int compareResult = value.compareTo(minValue.getDate());
        		if ( (!excludeMin && compareResult < 0) || (excludeMin && compareResult <= 0) ) { 
        			request.addErrorMessage("loom.validation.dateMinFailed", value);
        		}
        	}
        	if (maxValue != null) {
        		int compareResult = value.compareTo(maxValue.getDate());
        		if ( (!excludeMax && compareResult > 0) || (excludeMax && compareResult >= 0) ) {
        			request.addErrorMessage("loom.validation.dateMaxFailed", value);
        		}
        	}
    	}
    }
    
    @Override
    public String toString() {
    	return new ToStringBuilder(this)
    		.append("propertyPath", getPropertyPath())
    		.append("minValue", minValue)
    		.append("maxValue", maxValue)
    		.append("excludeMin", excludeMin)
    		.append("excludeMax", excludeMax)
    		.toString();
    }
 
    public void setMaxValue(DateValue maxValue) {
		this.maxValue = maxValue;
	}

	public void setMinValue(DateValue minValue) {
		this.minValue = minValue;
	}

	/**
     * Convert any kind of Date object to joda
     */
    protected DateTime toDateTime(Object v) {
		return new DateTime(v);
	}

    @Override
    public void decorate(FormInputTag t) {
    	super.decorate(t); 
    	if (t instanceof InputTextTag) {
    		InputTextTag tag = (InputTextTag) t;
    		if (minValue != null) {
    			tag.setMin(minValue.getDate());
    			tag.setExcludeMin(excludeMin);
    		}
    		if (maxValue != null) {
    			tag.setMax(maxValue.getDate());
    			tag.setExcludeMax(excludeMax);
    		}
    	}
    }

	public boolean isExcludeMax() {
		return excludeMax;
	}

	public void setExcludeMax(boolean excludeMax) {
		this.excludeMax = excludeMax;
	}

	public boolean isExcludeMin() {
		return excludeMin;
	}

	public void setExcludeMin(boolean excludeMin) {
		this.excludeMin = excludeMin;
	}

	public DateValue getMaxValue() {
		return maxValue;
	}

	public DateValue getMinValue() {
		return minValue;
	}

}
